@@ -10,7 +10,7 @@
  *
  * File begun on 2010-08-10 by RGerhards
  *
- * Copyright 2007-2016 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2007-2018 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
@@ -55,6 +55,7 @@
 #include <stdint.h>
 #include <zlib.h>
 #include <sys/stat.h>
+#include <regex.h>
 #if HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -131,6 +132,7 @@ struct instanceConf_s {
 	int bSuppOctetFram;		/* support octet-counted framing? */
 	int bSPFramingFix;
 	int iAddtlFrameDelim;
+	int socketBacklog;
 	sbool multiLine;
 	uint8_t compressionMode;
 	uchar *pszBindPort;		/* port to bind to */
@@ -150,6 +152,8 @@ struct instanceConf_s {
 	sbool flowControl;
 	int ratelimitInterval;
 	int ratelimitBurst;
+	uchar *startRegex;
+	regex_t end_preg;	/* compiled version of startRegex */
 	struct instanceConf_s *next;
 };
 
@@ -192,6 +196,7 @@ static struct cnfparamdescr inppdescr[] = {
 	{ "flowcontrol", eCmdHdlrBinary, 0 },
 	{ "name", eCmdHdlrString, 0 },
 	{ "maxframesize", eCmdHdlrInt, 0 },
+	{ "framing.delimiter.regex", eCmdHdlrString, 0 },
 	{ "ruleset", eCmdHdlrString, 0 },
 	{ "defaulttz", eCmdHdlrString, 0 },
 	{ "supportoctetcountedframing", eCmdHdlrBinary, 0 },
@@ -206,7 +211,8 @@ static struct cnfparamdescr inppdescr[] = {
 	{ "addtlframedelimiter", eCmdHdlrInt, 0 },
 	{ "ratelimit.interval", eCmdHdlrInt, 0 },
 	{ "ratelimit.burst", eCmdHdlrInt, 0 },
-	{ "multiline", eCmdHdlrBinary, 0 }
+	{ "multiline", eCmdHdlrBinary, 0 },
+	{ "socketbacklog", eCmdHdlrInt, 0 }
 };
 static struct cnfparamblk inppblk =
 	{ CNFPARAMBLK_VERSION,
@@ -238,6 +244,7 @@ struct ptcpsrv_s {
 	int maxFrameSize;
 	int	bFailOnPerms;	/* fail creation if chown fails? */
 	sbool bUnixSocket;
+	int socketBacklog;
 	int iAddtlFrameDelim;
 	sbool multiLine;
 	int iKeepAliveIntvl;
@@ -260,6 +267,7 @@ struct ptcpsrv_s {
 	sbool discardTruncatedMsg;
 	sbool flowControl;
 	ratelimit_t *ratelimiter;
+	instanceConf_t *inst;
 };
 
 /* the ptcp session object. Describes a single active session.
@@ -273,8 +281,8 @@ struct ptcpsess_s {
 	sbool bzInitDone; /* did we do an init of zstrm already? */
 	z_stream zstrm;	/* zip stream to use for tcp compression */
 	uint8_t compressionMode;
-//--- from tcps_sess.h
 	int iMsg;		 /* index of next char to store in msg */
+	int iCurrLine;		 /* 2nd char of current line in regex framing mode */
 	int bAtStrtOfFram;	/* are we at the very beginning of a new frame? */
 	sbool bSuppOctetFram;	/**< copy from listener, to speed up access */
 	sbool bSPFramingFix;
@@ -287,9 +295,9 @@ struct ptcpsess_s {
 	int iOctetsRemain;	/* Number of Octets remaining in message */
 	TCPFRAMINGMODE eFraming;
 	uchar *pMsg;		/* message (fragment) received */
+	uchar *pMsg_save;	/* message (fragment) save area in regex framing mode */
 	prop_t *peerName;	/* host name we received messages from */
 	prop_t *peerIP;
-//--- END from tcps_sess.h
 };
 
 
@@ -369,6 +377,7 @@ static rsRetVal addLstn(ptcpsrv_t *pSrv, int sock, int isIPv6);
 static void
 destructSess(ptcpsess_t *pSess)
 {
+	free(pSess->pMsg_save);
 	free(pSess->pMsg);
 	free(pSess->epd);
 	prop.Destruct(&pSess->peerName);
@@ -441,7 +450,7 @@ static rsRetVal startupUXSrv(ptcpsrv_t *pSrv) {
 		ABORT_FINALIZE(RS_RET_ERR_CRE_AFUX);
 	}
 
-	if (listen(sock, 5) < 0) {
+	if (listen(sock, pSrv->socketBacklog) < 0) {
 		errmsg.LogError(errno, RS_RET_ERR_CRE_AFUX, "imptcp: unix socket listen error");
 		ABORT_FINALIZE(RS_RET_ERR_CRE_AFUX);
 	}
@@ -593,7 +602,7 @@ startupSrv(ptcpsrv_t *pSrv)
 			continue;
 		}
 
-		if(listen(sock, 511) < 0) {
+		if(listen(sock, pSrv->socketBacklog) < 0) {
 			DBGPRINTF("tcp listen error %d, suspending\n", errno);
 			close(sock);
 			sock = -1;
@@ -910,6 +919,58 @@ finalize_it:
 }
 
 
+/* process the data received, special case if the framing is specified via
+ * a regex. For more info see processDataRcvd().
+ */
+static rsRetVal ATTR_NONNULL()
+processDataRcvd_regexFraming(ptcpsess_t *const __restrict__ pThis,
+	char **const buff,
+	struct syslogTime *const stTime,
+	const time_t ttGenTime,
+	multi_submit_t *const pMultiSub,
+	unsigned *const __restrict__ pnMsgs)
+{
+	DEFiRet;
+	const instanceConf_t *const inst = pThis->pLstn->pSrv->inst;
+	assert(inst->startRegex != NULL);
+	const char c = **buff;
+
+	pThis->pMsg[pThis->iMsg++] = c;
+	pThis->pMsg[pThis->iMsg] = '\0';
+
+	if(pThis->iMsg == 2*iMaxLine) {
+		LogError(0, RS_RET_OVERSIZE_MSG, "imptcp: more then double max message size (%d) "
+			"received without finding frame terminator via regex - assuming "
+			"end of frame now.", pThis->iMsg+1);
+		doSubmitMsg(pThis, stTime, ttGenTime, pMultiSub);
+		++(*pnMsgs);
+		pThis->iMsg = 0;
+		pThis->iCurrLine = 1;
+	}
+
+
+	if(c == '\n') {
+		pThis->iCurrLine = pThis->iMsg;
+	} else {
+		const int isMatch = !regexec(&inst->end_preg, (char*)pThis->pMsg+pThis->iCurrLine, 0, NULL, 0);
+		if(isMatch) {
+			DBGPRINTF("regex match (%d), framing line: %s\n", pThis->iCurrLine, pThis->pMsg);
+			strcpy((char*)pThis->pMsg_save, (char*) pThis->pMsg+pThis->iCurrLine);
+			pThis->iMsg = pThis->iCurrLine - 1;
+
+			doSubmitMsg(pThis, stTime, ttGenTime, pMultiSub);
+			++(*pnMsgs);
+
+			strcpy((char*)pThis->pMsg, (char*)pThis->pMsg_save);
+			pThis->iMsg = ustrlen(pThis->pMsg_save);
+			pThis->iCurrLine = 1;
+		}
+	}
+
+	RETiRet;
+}
+
+
 /* process the data received. As TCP is stream based, we need to process the
  * data inside a state machine. The actual data received is passed in byte-by-byte
  * from DataRcvd, and this function here compiles messages from them and submits
@@ -934,6 +995,11 @@ processDataRcvd(ptcpsess_t *const __restrict__ pThis,
 	uchar *propPeerIP = NULL;
 	int lenPeerIP = 0;
 
+	if(pThis->pLstn->pSrv->inst->startRegex != NULL) {
+		processDataRcvd_regexFraming(pThis, buff, stTime, ttGenTime, pMultiSub, pnMsgs);
+		FINALIZE;
+	}
+
 	if(pThis->inputState == eAtStrtFram) {
 		if(pThis->bSuppOctetFram && isdigit((int) c)) {
 			pThis->inputState = eInOctetCnt;
@@ -1344,15 +1410,25 @@ addSess(ptcplstn_t *pLstn, int sock, prop_t *peerName, prop_t *peerIP)
 	DEFiRet;
 	ptcpsess_t *pSess = NULL;
 	ptcpsrv_t *pSrv = pLstn->pSrv;
+	int pmsg_size_factor;
 
 	CHKmalloc(pSess = malloc(sizeof(ptcpsess_t)));
-	CHKmalloc(pSess->pMsg = malloc(iMaxLine));
+	if(pLstn->pSrv->inst->startRegex == NULL) {
+		pmsg_size_factor = 1;
+		pSess->pMsg_save = NULL;
+	} else {
+		pmsg_size_factor = 2;
+		pSess->pMsg = NULL;
+		CHKmalloc(pSess->pMsg_save = malloc(1 + iMaxLine * pmsg_size_factor));
+	}
+	CHKmalloc(pSess->pMsg = malloc(1 + iMaxLine * pmsg_size_factor));
 	pSess->pLstn = pLstn;
 	pSess->sock = sock;
 	pSess->bSuppOctetFram = pLstn->bSuppOctetFram;
 	pSess->bSPFramingFix = pLstn->bSPFramingFix;
 	pSess->inputState = eAtStrtFram;
 	pSess->iMsg = 0;
+	pSess->iCurrLine = 1;
 	pSess->bzInitDone = 0;
 	pSess->bAtStrtOfFram = 1;
 	pSess->peerName = peerName;
@@ -1373,8 +1449,8 @@ addSess(ptcplstn_t *pLstn, int sock, prop_t *peerName, prop_t *peerIP)
 finalize_it:
 	if(iRet != RS_RET_OK) {
 		if(pSess != NULL) {
-			if(pSess->pMsg != NULL)
-				free(pSess->pMsg);
+			free(pSess->pMsg_save);
+			free(pSess->pMsg);
 			free(pSess);
 		}
 	}
@@ -1500,11 +1576,13 @@ createInstance(instanceConf_t **pinst)
 	inst->bEmitMsgOnOpen = 0;
 	inst->dfltTZ = NULL;
 	inst->iAddtlFrameDelim = TCPSRV_NO_ADDTL_DELIMITER;
+	inst->startRegex = NULL;
 	inst->pBindRuleset = NULL;
 	inst->ratelimitBurst = 10000; /* arbitrary high limit */
 	inst->ratelimitInterval = 0; /* off */
 	inst->compressionMode = COMPRESS_SINGLE_MSG;
 	inst->multiLine = 0;
+	inst->socketBacklog = 5;
 
 	/* node created, let's add to config */
 	if(loadModConf->tail == NULL) {
@@ -1579,6 +1657,7 @@ addListner(modConfData_t __attribute__((unused)) *modConf, instanceConf_t *inst)
 	pthread_mutex_init(&pSrv->mutSessLst, NULL);
 	pSrv->pSess = NULL;
 	pSrv->pLstn = NULL;
+	pSrv->inst = inst;
 	pSrv->bSuppOctetFram = inst->bSuppOctetFram;
 	pSrv->bSPFramingFix = inst->bSPFramingFix;
 	pSrv->bKeepAlive = inst->bKeepAlive;
@@ -1594,6 +1673,7 @@ addListner(modConfData_t __attribute__((unused)) *modConf, instanceConf_t *inst)
 	}
 	pSrv->iAddtlFrameDelim = inst->iAddtlFrameDelim;
 	pSrv->multiLine = inst->multiLine;
+	pSrv->socketBacklog = inst->socketBacklog;
 	pSrv->maxFrameSize = inst->maxFrameSize;
 	if (inst->pszBindAddr == NULL) {
 		pSrv->lstnIP = NULL;
@@ -1958,9 +2038,10 @@ wrkr(void *myself)
 				pthread_mutex_unlock(&io_q.mut);
 				break;
 			} else {
-				DBGPRINTF("imptcp: worker %u waiting on new work items\n", (unsigned) me->tid);
+				DBGPRINTF("imptcp: worker %llu waiting on new work items\n",
+					(unsigned long long) me->tid);
 				pthread_cond_wait(&io_q.wakeup_worker, &io_q.mut);
-				DBGPRINTF("imptcp: worker %u awoken\n", (unsigned) me->tid);
+				DBGPRINTF("imptcp: worker %llu awoken\n", (unsigned long long) me->tid);
 			}
 			++wrkrRunning;
 		}
@@ -2038,6 +2119,8 @@ CODESTARTnewInpInst
 						"parameter given is %d, max is 200000000", max);
 				ABORT_FINALIZE(RS_RET_PARAM_ERROR);
 			}
+		} else if(!strcmp(inppblk.descr[i].name, "framing.delimiter.regex")) {
+			inst->startRegex = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(inppblk.descr[i].name, "ruleset")) {
 			inst->pszBindRuleset = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(inppblk.descr[i].name, "supportoctetcountedframing")) {
@@ -2079,6 +2162,8 @@ CODESTARTnewInpInst
 			inst->ratelimitInterval = (int) pvals[i].val.d.n;
 		} else if(!strcmp(inppblk.descr[i].name, "multiline")) {
 			inst->multiLine = (sbool) pvals[i].val.d.n;
+		} else if(!strcmp(inppblk.descr[i].name, "socketbacklog")) {
+			inst->socketBacklog = (int) pvals[i].val.d.n;
 		} else {
 			dbgprintf("imptcp: program error, non-handled "
 			  "param '%s'\n", inppblk.descr[i].name);
@@ -2091,6 +2176,16 @@ CODESTARTnewInpInst
 			ABORT_FINALIZE(RS_RET_PARAM_ERROR);
 		}
 	}
+
+	if(inst->startRegex != NULL) {
+		const int errcode = regcomp(&inst->end_preg, (char*)inst->startRegex, REG_EXTENDED);
+		if(errcode != 0) {
+			char errbuff[512];
+			regerror(errcode, &inst->end_preg, errbuff, sizeof(errbuff));
+			parser_errmsg("imptcp: error in framing.delimiter.regex expansion: %s", errbuff);
+			ABORT_FINALIZE(RS_RET_ERR);
+		}
+	}
 finalize_it:
 CODE_STD_FINALIZERnewInpInst
 	cnfparamvalsDestruct(pvals, &inppblk);
@@ -2242,6 +2337,10 @@ CODESTARTfreeCnf
 		free(inst->pszBindRuleset);
 		free(inst->pszInputName);
 		free(inst->dfltTZ);
+		if(inst->startRegex != NULL) {
+			regfree(&inst->end_preg);
+			free(inst->startRegex);
+		}
 		del = inst;
 		inst = inst->next;
 		free(del);
