@@ -60,6 +60,7 @@
 #include "datetime.h"
 #include "errmsg.h"
 #include "cfsysline.h"
+#include "parserif.h"
 #include "unicode-helper.h"
 
 MODULE_TYPE_OUTPUT
@@ -83,6 +84,8 @@ typedef struct _instanceData {
     	char *ssl_cert;
     	char *uid;
     	char *pwd;
+	uint32_t allowed_error_codes[256];
+	int allowed_error_codes_nbr;
    	char *db;
    	char *collection_name;
 	char *tplName;
@@ -106,7 +109,8 @@ static struct cnfparamdescr actpdescr[] = {
 	{ "pwd", eCmdHdlrGetWord, 0 },
 	{ "db", eCmdHdlrGetWord, 0 },
 	{ "collection", eCmdHdlrGetWord, 0 },
-	{ "template", eCmdHdlrGetWord, 0 }
+	{ "template", eCmdHdlrGetWord, 0 },
+	{ "allowed_error_codes", eCmdHdlrArray, 0 }
 };
 static struct cnfparamblk actpblk =
 	{ CNFPARAMBLK_VERSION,
@@ -503,6 +507,23 @@ CODESTARTtryResume
 	}
 ENDtryResume
 
+/*
+ * Check if `code` is in the allowed error codes.
+ * Return 1 if so, 0 otherwise.
+ */
+static int is_allowed_error_code(instanceData const* pData, uint32_t code)
+{
+	int i;
+
+	i = 0;
+	while (i < pData->allowed_error_codes_nbr) {
+		if (code == pData->allowed_error_codes[i])
+			return 1;
+		++i;
+	}
+	return 0;
+}
+
 BEGINdoAction_NoStrings
 	bson_t *doc = NULL;
 	instanceData *pData;
@@ -526,6 +547,8 @@ CODESTARTdoAction
 	}
 	if (mongoc_collection_insert (pData->collection, MONGOC_INSERT_NONE, doc, NULL, &(pData->error) ) ) {
 		pData->bErrMsgPermitted = 1;
+	} else if (is_allowed_error_code(pData, pData->error.code)) {
+		dbgprintf("ommongodb: insert error: allowing error code\n");
 	} else {
 		dbgprintf("ommongodb: insert error\n");
 		reportMongoError(pData);
@@ -553,6 +576,8 @@ static void setInstParamDefaults(instanceData *pData)
 	pData->db = NULL;
 	pData->collection = NULL;
 	pData->tplName = NULL;
+	memset (pData->allowed_error_codes, 0, 256 * sizeof(uint32_t));
+	pData->allowed_error_codes_nbr = 0;
 }
 
 BEGINnewActInst
@@ -592,6 +617,21 @@ CODESTARTnewActInst
 			pData->pwd = (char*)es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(actpblk.descr[i].name, "template")) {
 			pData->tplName = (char*)es_str2cstr(pvals[i].val.d.estr, NULL);
+		} else if(!strcmp(actpblk.descr[i].name, "allowed_error_codes")) {
+			const int maxerrcodes = sizeof(pData->allowed_error_codes) / sizeof(uint32_t);
+			pData->allowed_error_codes_nbr = pvals[i].val.d.ar->nmemb;
+			if(pData->allowed_error_codes_nbr > maxerrcodes) {
+				parser_errmsg("ommongodb: %d allowed_error_codes given, but max "
+					"supported number is %d. Only the first %d error codes will "
+					"be accepted", pData->allowed_error_codes_nbr, maxerrcodes, maxerrcodes);
+				pData->allowed_error_codes_nbr = maxerrcodes;
+			}
+			for(int j = 0 ; j <  pData->allowed_error_codes_nbr ; ++j) {
+				const char *const str = es_str2cstr(pvals[i].val.d.ar->arr[j], NULL);
+				assert(str != NULL);
+				pData->allowed_error_codes[j] = (unsigned)atoi(str);
+				free((void*)str);
+			}
 		} else {
 			dbgprintf("ommongodb: program error, non-handled "
 			  "param '%s'\n", actpblk.descr[i].name);
