@@ -48,17 +48,20 @@ MODULE_CNFNAME("omstdout")
 
 static rsRetVal resetConfigVariables(uchar __attribute__((unused)) *pp, void __attribute__((unused)) *pVal);
 
+/* static data */
+DEFobjCurrIf(errmsg);
+
 /* internal structures
  */
 DEF_OMOD_STATIC_DATA
 
 /* config variables */
 
-
 typedef struct _instanceData {
 	int bUseArrayInterface;		/* uses action use array instead of string template interface? */
 	int bEnsureLFEnding;		/* ensure that a linefeed is written at the end of EACH
 					record (test aid for nettester) */
+	uchar *templateName;
 } instanceData;
 
 typedef struct wrkrInstanceData {
@@ -67,10 +70,32 @@ typedef struct wrkrInstanceData {
 
 typedef struct configSettings_s {
 	int bUseArrayInterface;		/* shall action use array instead of string template interface? */
-	int bEnsureLFEnding;		/* shall action use array instead of string template interface? */
+	int bEnsureLFEnding;
+	int templateName;
 } configSettings_t;
 static configSettings_t cs;
 
+/* tables for interfacing with the v6 config system */
+/* action (instance) parameters */
+static struct cnfparamdescr actpdescr[] = {
+	{ "ensurelfending", eCmdHdlrBinary, 0 },
+	{ "template", eCmdHdlrGetWord, 0 }
+};
+static struct cnfparamblk actpblk =
+	{ CNFPARAMBLK_VERSION,
+	  sizeof(actpdescr)/sizeof(struct cnfparamdescr),
+	  actpdescr
+	};
+
+struct modConfData_s {
+	rsconf_t *pConf;	/* our overall config object */
+};
+
+static modConfData_t *loadModConf = NULL;/* modConf ptr to use for the current load process */
+static modConfData_t *runModConf = NULL;/* modConf ptr to use for the current exec process */
+
+
+
 BEGINinitConfVars		/* (re)set config variables to default values */
 CODESTARTinitConfVars 
 	resetConfigVariables(NULL, NULL);
@@ -86,6 +111,32 @@ CODESTARTcreateWrkrInstance
 ENDcreateWrkrInstance
 
 
+BEGINbeginCnfLoad
+CODESTARTbeginCnfLoad
+	loadModConf = pModConf;
+	pModConf->pConf = pConf;
+ENDbeginCnfLoad
+
+
+BEGINendCnfLoad
+CODESTARTendCnfLoad
+	loadModConf = NULL; /* done loading */
+ENDendCnfLoad
+
+BEGINcheckCnf
+CODESTARTcheckCnf
+ENDcheckCnf
+
+BEGINactivateCnf
+CODESTARTactivateCnf
+	runModConf = pModConf;
+ENDactivateCnf
+
+BEGINfreeCnf
+CODESTARTfreeCnf
+ENDfreeCnf
+
+
 BEGINisCompatibleWithFeature
 CODESTARTisCompatibleWithFeature
 	if(eFeat == sFEATURERepeatedMsgReduction)
@@ -105,6 +156,9 @@ ENDfreeWrkrInstance
 
 BEGINdbgPrintInstInfo
 CODESTARTdbgPrintInstInfo
+	dbgprintf("omstdout\n");
+	dbgprintf("\tensureLFEnding='%d'\n", pData->bEnsureLFEnding);
+	dbgprintf("\ttemplate='%s'\n", pData->templateName);
 ENDdbgPrintInstInfo
 
 
@@ -122,7 +176,9 @@ BEGINdoAction
 	size_t len;
 	int r;
 CODESTARTdoAction
+dbgprintf("omstdout: in doAction\n");
 	if(pWrkrData->pData->bUseArrayInterface) {
+dbgprintf("omstdout: in ArrayInterface\n");
 		/* if we use array passing, we need to put together a string
 		 * ourselves. At this point, please keep in mind that omstdout is
 		 * primarily a testing aid. Other modules may do different processing
@@ -149,6 +205,7 @@ CODESTARTdoAction
 		szBuf[iBuf] = '\0';
 		toWrite = szBuf;
 	} else {
+dbgprintf("omstdout: in else\n");
 		toWrite = (char*) ppString[0];
 	}
 	len = strlen(toWrite);
@@ -156,6 +213,7 @@ CODESTARTdoAction
 	 * actually intends to use this module in production (why???), this code
 	 * needs to be more solid. -- rgerhards, 2012-11-28
 	 */
+dbgprintf("omstdout: len: %d, toWrite: %s\n", (int) len, toWrite);
 	if((r = write(1, toWrite, len)) != (int) len) { /* 1 is stdout! */
 		DBGPRINTF("omstdout: error %d writing to stdout[%zd]: %s\n",
 			r, len, toWrite);
@@ -168,6 +226,64 @@ CODESTARTdoAction
 	}
 ENDdoAction
 
+static void
+setInstParamDefaults(instanceData *pData)
+{
+	pData->bEnsureLFEnding = 1;
+	pData->templateName = (uchar*) "RSYSLOG_FileFormat";
+	pData->bUseArrayInterface = 0;
+}
+
+
+BEGINnewActInst
+	struct cnfparamvals *pvals;
+	int i;
+	int bDestructPValsOnExit;
+	uchar *tplToUse;
+CODESTARTnewActInst
+	DBGPRINTF("newActInst (omstdout)\n");
+
+	bDestructPValsOnExit = 0;
+	pvals = nvlstGetParams(lst, &actpblk, NULL);
+	if(pvals == NULL) {
+		errmsg.LogError(0, RS_RET_MISSING_CNFPARAMS, "omstdout: error reading "
+				"config parameters");
+		ABORT_FINALIZE(RS_RET_MISSING_CNFPARAMS);
+	}
+	bDestructPValsOnExit = 1;
+
+	if(Debug) {
+		dbgprintf("action param blk in omstdout:\n");
+		cnfparamsPrint(&actpblk, pvals);
+	}
+
+	CHKiRet(createInstance(&pData));
+	setInstParamDefaults(pData);
+
+	for(i = 0 ; i < actpblk.nParams ; ++i) {
+		if(!pvals[i].bUsed) {
+			continue;
+		} else if(!strcmp(actpblk.descr[i].name, "ensurelfending")) {
+			pData->bEnsureLFEnding = (int) pvals[i].val.d.n;
+		} else if(!strcmp(actpblk.descr[i].name, "template")) {
+			pData->templateName = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
+		} else {
+			DBGPRINTF("omstdout: program error, non-handled "
+			  "param '%s'\n", actpblk.descr[i].name);
+		}
+	}
+
+
+	CODE_STD_STRING_REQUESTnewActInst(1)
+	//TODO: make the template a parameter
+	tplToUse = (uchar*) strdup((pData->templateName == NULL) ? "RSYSLOG_FileFormat" : (char *)pData->templateName);
+	CHKiRet(OMSRsetEntry(*ppOMSR, 0, tplToUse, OMSR_NO_RQD_TPL_OPTS));
+CODE_STD_FINALIZERnewActInst
+	if(bDestructPValsOnExit)
+		cnfparamvalsDestruct(pvals, &actpblk);
+ENDnewActInst
+
+
 
 BEGINparseSelectorAct
 	int iTplOpts;
@@ -195,6 +311,7 @@ ENDparseSelectorAct
 
 BEGINmodExit
 CODESTARTmodExit
+	objRelease(errmsg, CORE_COMPONENT);
 ENDmodExit
 
 
@@ -202,7 +319,9 @@ BEGINqueryEtryPt
 CODESTARTqueryEtryPt
 CODEqueryEtryPt_STD_OMOD_QUERIES
 CODEqueryEtryPt_STD_OMOD8_QUERIES
-CODEqueryEtryPt_STD_CONF2_CNFNAME_QUERIES 
+CODEqueryEtryPt_STD_CONF2_CNFNAME_QUERIES
+CODEqueryEtryPt_STD_CONF2_QUERIES
+CODEqueryEtryPt_STD_CONF2_OMOD_QUERIES
 ENDqueryEtryPt
 
 
@@ -245,6 +364,7 @@ CODEmodInit_QueryRegCFSLineHdlr
 		CHKiRet(omsdRegCFSLineHdlr((uchar *)"actionomstdoutarrayinterface", 0, eCmdHdlrBinary, NULL,
 			                   &cs.bUseArrayInterface, STD_LOADABLE_MODULE_ID));
 	}
+	CHKiRet(objUse(errmsg, CORE_COMPONENT));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"actionomstdoutensurelfending", 0, eCmdHdlrBinary, NULL,
 				   &cs.bEnsureLFEnding, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler,
