@@ -40,6 +40,7 @@
  */
 #ifdef HAVE_ATOMIC_BUILTINS
 #	define ATOMIC_SUB(data, val, phlpmut) __sync_fetch_and_sub(data, val)
+#	define ATOMIC_SUB_unsigned(data, val, phlpmut) __sync_fetch_and_sub(data, val)
 #	define ATOMIC_ADD(data, val) __sync_fetch_and_add(&(data), val)
 #	define ATOMIC_INC(data, phlpmut) ((void) __sync_fetch_and_add(data, 1))
 #	define ATOMIC_INC_AND_FETCH_int(data, phlpmut) __sync_fetch_and_add(data, 1)
@@ -47,6 +48,7 @@
 #	define ATOMIC_DEC(data, phlpmut) ((void) __sync_sub_and_fetch(data, 1))
 #	define ATOMIC_DEC_AND_FETCH(data, phlpmut) __sync_sub_and_fetch(data, 1)
 #	define ATOMIC_FETCH_32BIT(data, phlpmut) ((int) __sync_fetch_and_and(data, 0xffffffff))
+#	define ATOMIC_FETCH_32BIT_unsigned(data, phlpmut) ((int) __sync_fetch_and_and(data, 0xffffffff))
 #	define ATOMIC_STORE_1_TO_32BIT(data) __sync_lock_test_and_set(&(data), 1)
 #	define ATOMIC_STORE_0_TO_INT(data, phlpmut) __sync_fetch_and_and(data, 0)
 #	define ATOMIC_STORE_1_TO_INT(data, phlpmut) __sync_fetch_and_or(data, 1)
@@ -182,12 +184,28 @@
 		return(val);
 	}
 
+	static inline int
+	ATOMIC_FETCH_32BIT_unsigned(unsigned *data, pthread_mutex_t *phlpmut) {
+		int val;
+		pthread_mutex_lock(phlpmut);
+		val = (*data);
+		pthread_mutex_unlock(phlpmut);
+		return(val);
+	}
+
 	static inline void
 	ATOMIC_SUB(int *data, int val, pthread_mutex_t *phlpmut) {
 		pthread_mutex_lock(phlpmut);
 		(*data) -= val;
 		pthread_mutex_unlock(phlpmut);
 	}
+
+	static inline void
+	ATOMIC_SUB_unsigned(unsigned *data, int val, pthread_mutex_t *phlpmut) {
+		pthread_mutex_lock(phlpmut);
+		(*data) -= val;
+		pthread_mutex_unlock(phlpmut);
+	}
 #	define DEF_ATOMIC_HELPER_MUT(x)  pthread_mutex_t x;
 #	define INIT_ATOMIC_HELPER_MUT(x) pthread_mutex_init(&(x), NULL);
 #	define DESTROY_ATOMIC_HELPER_MUT(x) pthread_mutex_destroy(&(x));
