@@ -7,7 +7,7 @@
  *
  * Module begun 2008-04-16 by Rainer Gerhards
  *
- * Copyright 2008-2017 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2018 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of the rsyslog runtime library.
  *
@@ -62,7 +62,6 @@
 /* static data */
 DEFobjStaticHelpers
 DEFobjCurrIf(prop)
-DEFobjCurrIf(errmsg)
 DEFobjCurrIf(net)
 
 /* static data
@@ -86,6 +85,9 @@ static uchar *stdlog_chanspec = NULL;
 static int bParseHOSTNAMEandTAG = 1;	/* parser modification (based on startup params!) */
 static int bPreserveFQDN = 0;		/* should FQDNs always be preserved? */
 static int iMaxLine = 8096;		/* maximum length of a syslog message */
+static uchar * oversizeMsgErrorFile = NULL;		/* File where oversize messages are written to */
+static int oversizeMsgInputMode = 0;	/* Mode which oversize messages will be forwarded */
+static int reportOversizeMsg = 1;	/* shall error messages be generated for oversize messages? */
 static int iGnuTLSLoglevel = 0;
 static int iDefPFFamily = PF_UNSPEC;     /* protocol family (IPv4, IPv6 or both) */
 static int bDropMalPTRMsgs = 0;/* Drop messages which have malicious PTR records during DNS lookup */
@@ -127,6 +129,9 @@ int glblIntMsgRateLimitBurst = 500;
 char** glblDbgFiles = NULL;
 size_t glblDbgFilesNum = 0;
 int glblDbgWhitelist = 1;
+int glblPermitCtlC = 0;
+
+uint64_t glblDevOptions = 0; /* to be used by developers only */
 
 pid_t glbl_ourpid;
 #ifndef HAVE_ATOMIC_BUILTINS
@@ -155,6 +160,9 @@ static struct cnfparamdescr cnfparamdescr[] = {
         { "defaultnetstreamdrivercertfile", eCmdHdlrString, 0 },
 	{ "defaultnetstreamdriver", eCmdHdlrString, 0 },
 	{ "maxmessagesize", eCmdHdlrSize, 0 },
+	{ "oversizemsg.errorfile", eCmdHdlrGetWord, 0 },
+	{ "oversizemsg.report", eCmdHdlrBinary, 0 },
+	{ "oversizemsg.input.mode", eCmdHdlrGetWord, 0 },
 	{ "action.reportsuspension", eCmdHdlrBinary, 0 },
 	{ "action.reportsuspensioncontinuation", eCmdHdlrBinary, 0 },
 	{ "parser.controlcharacterescapeprefix", eCmdHdlrGetChar, 0 },
@@ -182,9 +190,11 @@ static struct cnfparamdescr cnfparamdescr[] = {
 	{ "environment", eCmdHdlrArray, 0 },
 	{ "processinternalmessages", eCmdHdlrBinary, 0 },
 	{ "umask", eCmdHdlrFileCreateMode, 0 },
+	{ "internal.developeronly.options", eCmdHdlrInt, 0 },
 	{ "internalmsg.ratelimit.interval", eCmdHdlrPositiveInt, 0 },
 	{ "internalmsg.ratelimit.burst", eCmdHdlrPositiveInt, 0 },
 	{ "errormessagestostderr.maxnumber", eCmdHdlrPositiveInt, 0 },
+	{ "shutdown.enable.ctlc", eCmdHdlrBinary, 0 },
 	{ "debug.files", eCmdHdlrArray, 0 },
 	{ "debug.whitelist", eCmdHdlrBinary, 0 }
 };
@@ -216,6 +226,7 @@ glblGetMaxLine(void)
 	return(iMaxLine);
 }
 
+
 int
 GetGnuTLSLoglevel(void)
 {
@@ -325,14 +336,14 @@ setLocalHostIPIF(void __attribute__((unused)) *pVal, uchar *pNewVal)
 	CHKiRet(objUse(net, CORE_COMPONENT));
 
 	if(propLocalIPIF_set) {
-		errmsg.LogError(0, RS_RET_ERR, "$LocalHostIPIF is already set "
+		LogError(0, RS_RET_ERR, "$LocalHostIPIF is already set "
 				"and cannot be reset; place it at TOP OF rsyslog.conf!");
 		ABORT_FINALIZE(RS_RET_ERR);
 	}
 
 	localRet = net.GetIFIPAddr(pNewVal, AF_UNSPEC, myIP, (int) sizeof(myIP));
 	if(localRet != RS_RET_OK) {
-		errmsg.LogError(0, RS_RET_ERR, "$LocalHostIPIF: IP address for interface "
+		LogError(0, RS_RET_ERR, "$LocalHostIPIF: IP address for interface "
 				"'%s' cannnot be obtained - ignoring directive", pNewVal);
 	} else  {
 		storeLocalHostIPIF(myIP);
@@ -365,25 +376,25 @@ static rsRetVal setWorkDir(void __attribute__((unused)) *pVal, uchar *pNewVal)
 	}
 
 	if(i < 0) {
-		errmsg.LogError(0, RS_RET_ERR_WRKDIR, "$WorkDirectory: empty value "
+		LogError(0, RS_RET_ERR_WRKDIR, "$WorkDirectory: empty value "
 				"- directive ignored");
 		ABORT_FINALIZE(RS_RET_ERR_WRKDIR);
 	}
 
 	if(i != (int) lenDir - 1) {
 		pNewVal[i+1] = '\0';
-		errmsg.LogError(0, RS_RET_WRN_WRKDIR, "$WorkDirectory: trailing slashes "
+		LogError(0, RS_RET_WRN_WRKDIR, "$WorkDirectory: trailing slashes "
 			"removed, new value is '%s'", pNewVal);
 	}
 
 	if(stat((char*) pNewVal, &sb) != 0) {
-		errmsg.LogError(0, RS_RET_ERR_WRKDIR, "$WorkDirectory: %s can not be "
+		LogError(0, RS_RET_ERR_WRKDIR, "$WorkDirectory: %s can not be "
 				"accessed, probably does not exist - directive ignored", pNewVal);
 		ABORT_FINALIZE(RS_RET_ERR_WRKDIR);
 	}
 
 	if(!S_ISDIR(sb.st_mode)) {
-		errmsg.LogError(0, RS_RET_ERR_WRKDIR, "$WorkDirectory: %s not a directory - directive ignored", 
+		LogError(0, RS_RET_ERR_WRKDIR, "$WorkDirectory: %s not a directory - directive ignored", 
 				pNewVal);
 		ABORT_FINALIZE(RS_RET_ERR_WRKDIR);
 	}
@@ -401,12 +412,12 @@ static void
 setMaxLine(const int64_t iNew)
 {
 	if(iNew < 128) {
-		errmsg.LogError(0, RS_RET_INVALID_VALUE, "maxMessageSize tried to set "
+		LogError(0, RS_RET_INVALID_VALUE, "maxMessageSize tried to set "
 				"to %lld, but cannot be less than 128 - set to 128 "
 				"instead", (long long) iNew);
 		iMaxLine = 128;
 	} else if(iNew > (int64_t) INT_MAX) {
-		errmsg.LogError(0, RS_RET_INVALID_VALUE, "maxMessageSize larger than "
+		LogError(0, RS_RET_INVALID_VALUE, "maxMessageSize larger than "
 				"INT_MAX (%d) - reduced to INT_MAX", INT_MAX);
 		iMaxLine = INT_MAX;
 	} else {
@@ -414,6 +425,8 @@ setMaxLine(const int64_t iNew)
 	}
 }
 
+
+
 static rsRetVal
 legacySetMaxMessageSize(void __attribute__((unused)) *pVal, int64_t iNew)
 {
@@ -440,6 +453,22 @@ setDebugLevel(void __attribute__((unused)) *pVal, int level)
 	RETiRet;
 }
 
+static rsRetVal ATTR_NONNULL()
+setOversizeMsgInputMode(const uchar *const mode)
+{
+	DEFiRet;
+	if(!strcmp((char*)mode, "truncate")) {
+		oversizeMsgInputMode = glblOversizeMsgInputMode_Truncate;
+	} else if(!strcmp((char*)mode, "split")) {
+		oversizeMsgInputMode = glblOversizeMsgInputMode_Split;
+	} else if(!strcmp((char*)mode, "accept")) {
+		oversizeMsgInputMode = glblOversizeMsgInputMode_Accept;
+	} else {
+		oversizeMsgInputMode = glblOversizeMsgInputMode_Truncate;
+	}
+	RETiRet;
+}
+
 static rsRetVal
 setDisableDNS(int val)
 {
@@ -547,6 +576,29 @@ done:
 }
 
 
+/* return the name of the file where oversize messages are written to
+ */
+uchar*
+glblGetOversizeMsgErrorFile(void)
+{
+	return oversizeMsgErrorFile;
+}
+
+
+/* return the mode with which oversize messages will be put forward
+ */
+int
+glblGetOversizeMsgInputMode(void)
+{
+	return oversizeMsgInputMode;
+}
+
+int
+glblReportOversizeMessage(void)
+{
+	return reportOversizeMsg;
+}
+
 /* set our local domain name. Free previous domain, if it was already set.
  */
 static rsRetVal
@@ -795,6 +847,9 @@ static rsRetVal resetConfigVariables(uchar __attribute__((unused)) *pp, void __a
 	pszDfltNetstrmDrvrCertFile = NULL;
 	free(LocalHostNameOverride);
 	LocalHostNameOverride = NULL;
+	free(oversizeMsgErrorFile);
+	oversizeMsgErrorFile = NULL;
+	oversizeMsgInputMode = glblOversizeMsgInputMode_Accept;
 	free(pszWorkDir);
 	pszWorkDir = NULL;
 	bDropMalPTRMsgs = 0;
@@ -802,6 +857,7 @@ static rsRetVal resetConfigVariables(uchar __attribute__((unused)) *pp, void __a
 	iMaxLine = 8192;
 	cCCEscapeChar = '#';
 	bDropTrailingLF = 1;
+	reportOversizeMsg = 1;
 	bEscapeCCOnRcv = 1; /* default is to escape control characters */
 	bSpaceLFOnRcv = 0;
 	bEscape8BitChars = 0; /* default is not to escape control characters */
@@ -983,7 +1039,7 @@ glblProcessCnf(struct cnfobj *o)
 
 	cnfparamvals = nvlstGetParams(o->nvlst, &paramblk, cnfparamvals);
 	if(cnfparamvals == NULL) {
-		errmsg.LogError(0, RS_RET_MISSING_CNFPARAMS, "error processing global "
+		LogError(0, RS_RET_MISSING_CNFPARAMS, "error processing global "
 				"config parameters [global(...)]");
 		goto done;
 	}
@@ -1001,9 +1057,11 @@ glblProcessCnf(struct cnfobj *o)
 			continue;
 		if(!strcmp(paramblk.descr[i].name, "processinternalmessages")) {
 			bProcessInternalMessages = (int) cnfparamvals[i].val.d.n;
+		} else if(!strcmp(paramblk.descr[i].name, "internal.developeronly.options")) {
+		        glblDevOptions = (uint64_t) cnfparamvals[i].val.d.n;
 		} else if(!strcmp(paramblk.descr[i].name, "stdlog.channelspec")) {
 #ifndef HAVE_LIBLOGGING_STDLOG
-			errmsg.LogError(0, RS_RET_ERR, "rsyslog wasn't "
+			LogError(0, RS_RET_ERR, "rsyslog wasn't "
 				"compiled with liblogging-stdlog support. "
 				"The 'stdlog.channelspec' parameter "
 				"is ignored. Note: the syslog API is used instead.\n");
@@ -1029,7 +1087,7 @@ glblProcessMainQCnf(struct cnfobj *o)
 	if(mainqCnfObj == NULL) {
 		mainqCnfObj = o;
 	} else {
-		errmsg.LogError(0, RS_RET_ERR, "main_queue() object can only be specified "
+		LogError(0, RS_RET_ERR, "main_queue() object can only be specified "
 				"once - all but first ignored\n");
 	}
 }
@@ -1075,7 +1133,7 @@ do_setenv(const char *const var)
 	for(i = 0 ; *val != '=' ; ++i, ++val) {
 		if(i == sizeof(varname)-i) {
 			parser_errmsg("environment variable name too long "
-				"[max %zd chars] or malformed entry: '%s'",
+				"[max %zu chars] or malformed entry: '%s'",
 				sizeof(varname)-1, var);
 			ABORT_FINALIZE(RS_RET_ERR_SETENV);
 		}
@@ -1191,9 +1249,18 @@ glblDoneLoadCnf(void)
 			bActionReportSuspensionCont = (int) cnfparamvals[i].val.d.n;
 		} else if(!strcmp(paramblk.descr[i].name, "maxmessagesize")) {
 			setMaxLine(cnfparamvals[i].val.d.n);
+		} else if(!strcmp(paramblk.descr[i].name, "oversizemsg.errorfile")) {
+			free(oversizeMsgErrorFile);
+			oversizeMsgErrorFile = (uchar*)es_str2cstr(cnfparamvals[i].val.d.estr, NULL);
+		} else if(!strcmp(paramblk.descr[i].name, "oversizemsg.report")) {
+			reportOversizeMsg = (int) cnfparamvals[i].val.d.n;
+		} else if(!strcmp(paramblk.descr[i].name, "oversizemsg.input.mode")) {
+			const char *const tmp = es_str2cstr(cnfparamvals[i].val.d.estr, NULL);
+			setOversizeMsgInputMode((uchar*) tmp);
+			free((void*)tmp);
 		} else if(!strcmp(paramblk.descr[i].name, "debug.onshutdown")) {
 			glblDebugOnShutdown = (int) cnfparamvals[i].val.d.n;
-			errmsg.LogError(0, RS_RET_OK, "debug: onShutdown set to %d", glblDebugOnShutdown);
+			LogError(0, RS_RET_OK, "debug: onShutdown set to %d", glblDebugOnShutdown);
 		} else if(!strcmp(paramblk.descr[i].name, "debug.gnutls")) {
 			iGnuTLSLoglevel = (int) cnfparamvals[i].val.d.n;
 		} else if(!strcmp(paramblk.descr[i].name, "debug.unloadmodules")) {
@@ -1227,11 +1294,11 @@ glblDoneLoadCnf(void)
 				}
 				if((altdbg = open(pszAltDbgFileName, O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY
 				|O_CLOEXEC, S_IRUSR|S_IWUSR)) == -1) {
-					errmsg.LogError(0, RS_RET_ERR, "debug log file '%s' could not be opened",
+					LogError(0, RS_RET_ERR, "debug log file '%s' could not be opened",
 							pszAltDbgFileName);
 				}
 			}
-			errmsg.LogError(0, RS_RET_OK, "debug log file is '%s', fd %d", pszAltDbgFileName, altdbg);
+			LogError(0, RS_RET_OK, "debug log file is '%s', fd %d", pszAltDbgFileName, altdbg);
 		} else if(!strcmp(paramblk.descr[i].name, "janitor.interval")) {
 			janitorInterval = (int) cnfparamvals[i].val.d.n;
 		} else if(!strcmp(paramblk.descr[i].name, "net.ipprotocol")) {
@@ -1243,7 +1310,7 @@ glblDoneLoadCnf(void)
 			} else if(!strcmp(proto, "ipv6-only")) {
 				iDefPFFamily = PF_INET6;
 			} else{
-				errmsg.LogError(0, RS_RET_ERR, "invalid net.ipprotocol "
+				LogError(0, RS_RET_ERR, "invalid net.ipprotocol "
 					"parameter '%s' -- ignored", proto);
 			}
 			free(proto);
@@ -1290,6 +1357,8 @@ glblDoneLoadCnf(void)
 		        glblDbgWhitelist = (int) cnfparamvals[i].val.d.n;
 		} else if(!strcmp(paramblk.descr[i].name, "umask")) {
 		        loadConf->globals.umask = (int) cnfparamvals[i].val.d.n;
+		} else if(!strcmp(paramblk.descr[i].name, "shutdown.enable.ctlc")) {
+		        glblPermitCtlC = (int) cnfparamvals[i].val.d.n;
 		} else {
 			dbgprintf("glblDoneLoadCnf: program error, non-handled "
 			  "param '%s'\n", paramblk.descr[i].name);
@@ -1312,7 +1381,6 @@ finalize_it:	RETiRet;
 BEGINAbstractObjClassInit(glbl, 1, OBJ_IS_CORE_MODULE) /* class, version */
 	/* request objects we use */
 	CHKiRet(objUse(prop, CORE_COMPONENT));
-	CHKiRet(objUse(errmsg, CORE_COMPONENT));
 
 	/* intialize properties */
 	storeLocalHostIPIF((uchar*)"127.0.0.1");
@@ -1368,6 +1436,7 @@ BEGINObjClassExit(glbl, OBJ_IS_CORE_MODULE) /* class, version */
 	free(LocalDomain);
 	free(LocalHostName);
 	free(LocalHostNameOverride);
+	free(oversizeMsgErrorFile);
 	free(LocalFQDNName);
 	freeTimezoneInfo();
 	objRelease(prop, CORE_COMPONENT);
