@@ -50,7 +50,6 @@
 #include <fcntl.h>
 #include <stdarg.h>
 #include <limits.h>
-#define MAXFNAME 1024
 
 #include <ksi/ksi.h>
 #include <ksi/tlv_element.h>
@@ -58,10 +57,11 @@
 #include <ksi/net_async.h>
 #include <ksi/net_uri.h>
 #include <ksi/signature_builder.h>
+#include "rsyslog.h"
+#include "errmsg.h"
 #include "lib_ksils12.h"
 #include "lib_ksi_queue.h"
 
-typedef unsigned char uchar;
 #ifndef VERSION
 #define VERSION "no-version"
 #endif
@@ -487,16 +487,25 @@ ksiCloseSigFile(ksifile ksi) {
 
 static int mkpath(char* path, mode_t mode, uid_t uid, gid_t gid) {
 
+	if(path == NULL)
+		return 1;
+
 	for (char *p = strchr(path + 1, '/'); p; p = strchr(p + 1, '/')) {
 		*p = '\0';
-		if (mkdir(path, mode) == -1) {
-			if (errno != EEXIST) {
-				*p = '/';
-				return -1;
-			}
-			if (chown(p, uid, gid)) {
+		if (mkdir(path, mode) == 0) {
+			if (uid != (uid_t) -1 || gid != (uid_t) -1) {
+				if (chown(path, uid, gid)) {
+					LogError(errno, RS_RET_IO_ERROR,
+						"ksils12 signatures: could not change to "
+						"configured owner - files may be unaccessible");
+				}
 			}
 		}
+		else if (errno != EEXIST) {
+			*p = '/';
+			return -1;
+		}
+
 		*p = '/';
 	}
 	return 0;
@@ -509,6 +518,9 @@ ksiCreateFile(rsksictx ctx, const char *path, uid_t uid, gid_t gid, int mode, bo
 	FILE *f = NULL;
 	struct flock lock = {F_WRLCK, SEEK_SET, 0, 0, 0};
 
+	if(path ==NULL)
+		return NULL;
+
 	if (mkpath((char*) path, ctx->fDirCreateMode, ctx->dirUID, ctx->dirGID) != 0) {
 		report(ctx, "ksiCreateFile: mkpath failed for %s", path);
 		goto done;
@@ -546,8 +558,13 @@ ksiCreateFile(rsksictx ctx, const char *path, uid_t uid, gid_t gid, int mode, bo
 		goto done;
 	}
 
-	if (stat_st.st_size == 0 && header != NULL)
-		fwrite(header, strlen(header), 1, f);
+	if (stat_st.st_size == 0 && header != NULL) {
+		if(fwrite(header, strlen(header), 1, f) != 1) {
+        	        report(ctx, "ksiOpenSigFile: fwrite for file %s failed: %s",
+				path, strerror(errno));
+	                goto done;
+		}
+	}
 
 done:
 	return f;
@@ -1232,6 +1249,9 @@ static void process_requests(rsksictx ctx, KSI_CTX *ksi_ctx, FILE* outfile) {
 	int r = KSI_OK;
 	KSI_Signature *sig = NULL;
 
+	if(ProtectedQueue_count(ctx->signer_queue) == 0)
+		return;
+
 	while (ProtectedQueue_peekFront(ctx->signer_queue, (void**) &item) && item->type == QITEM_SIGNATURE_REQUEST) {
 		if (lastItem != NULL) {
 			if (outfile)
@@ -1254,6 +1274,14 @@ static void process_requests(rsksictx ctx, KSI_CTX *ksi_ctx, FILE* outfile) {
 	if (!outfile)
 		goto signing_failed;
 
+	if(!lastItem || item->type != QITEM_SIGNATURE_REQUEST)
+		return;
+
+	if(lastItem->arg == NULL) {
+		r = KSI_INVALID_ARGUMENT;
+		goto signing_failed;
+	}
+
 	r = KSI_Signature_signAggregated(ksi_ctx, lastItem->arg, lastItem->intarg2, &sig);
 	if (r != KSI_OK) {
 		reportKSIAPIErr(ctx, NULL, "KSI_Signature_createAggregated", r);
