@@ -33,6 +33,7 @@
  */
 #include "config.h"
 #include "rsyslog.h"
+#include "rainerscript.h"
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -63,7 +64,6 @@
 
 /* static data */
 DEFobjStaticHelpers
-DEFobjCurrIf(errmsg)
 DEFobjCurrIf(strgen)
 
 static modInfo_t *pLoadedModules = NULL;	/* list of currently-loaded modules */
@@ -112,7 +112,7 @@ static rsRetVal
 dummynewActInst(uchar *modName, struct nvlst __attribute__((unused)) *dummy1,
 	  	void __attribute__((unused)) **dummy2, omodStringRequest_t __attribute__((unused)) **dummy3) 
 {
-	errmsg.LogError(0, RS_RET_CONFOBJ_UNSUPPORTED, "config objects are not "
+	LogError(0, RS_RET_CONFOBJ_UNSUPPORTED, "config objects are not "
 			"supported by module '%s' -- legacy config options "
 			"MUST be used instead", modName);
 	return RS_RET_CONFOBJ_UNSUPPORTED;
@@ -379,7 +379,7 @@ readyModForCnf(modInfo_t *pThis, cfgmodules_etry_t **ppNew, cfgmodules_etry_t **
 			if(pLast->pMod == pThis) {
 				DBGPRINTF("module '%s' already in this config\n", modGetName(pThis));
 				if(strncmp((char*)modGetName(pThis), "builtin:", sizeof("builtin:")-1)) {
-					errmsg.LogError(0, RS_RET_MODULE_ALREADY_IN_CONF,
+					LogError(0, RS_RET_MODULE_ALREADY_IN_CONF,
 					   "module '%s' already in this config, cannot be added\n", modGetName(pThis));
 					ABORT_FINALIZE(RS_RET_MODULE_ALREADY_IN_CONF);
 				}
@@ -720,7 +720,7 @@ doModInit(rsRetVal (*modInit)(int, int*, rsRetVal(**)(), rsRetVal(*)(), modInfo_
 			}
 
 			if(pNew->mod.om.doAction == NULL && pNew->mod.om.commitTransaction == NULL) {
-				errmsg.LogError(0, RS_RET_INVLD_OMOD,
+				LogError(0, RS_RET_INVLD_OMOD,
 					"module %s does neither provide doAction() "
 					"nor commitTransaction() interface - cannot "
 					"load", name);
@@ -729,14 +729,14 @@ doModInit(rsRetVal (*modInit)(int, int*, rsRetVal(**)(), rsRetVal(*)(), modInfo_
 
 			if(pNew->mod.om.commitTransaction != NULL) {
 				if(pNew->mod.om.doAction != NULL){
-					errmsg.LogError(0, RS_RET_INVLD_OMOD,
+					LogError(0, RS_RET_INVLD_OMOD,
 						"module %s provides both doAction() "
 						"and commitTransaction() interface, using "
 						"commitTransaction()", name);
 					pNew->mod.om.doAction = NULL;
 				}
 				if(pNew->mod.om.beginTransaction == NULL){
-					errmsg.LogError(0, RS_RET_INVLD_OMOD,
+					LogError(0, RS_RET_INVLD_OMOD,
 						"module %s provides both commitTransaction() "
 						"but does not provide beginTransaction() - "
 						"cannot load", name);
@@ -803,6 +803,14 @@ doModInit(rsRetVal (*modInit)(int, int*, rsRetVal(**)(), rsRetVal(*)(), modInfo_
 			CHKiRet(strgen.SetModPtr(pStrgen, pNew));
 			CHKiRet(strgen.ConstructFinalize(pStrgen));
 			break;
+		case eMOD_FUNCTION:
+			CHKiRet((*pNew->modQueryEtryPt)((uchar*)"getFunctArray", &pNew->mod.fm.getFunctArray));
+			int version;
+			struct scriptFunct *functArray;
+			pNew->mod.fm.getFunctArray(&version, &functArray);
+			dbgprintf("LLL: %s\n", functArray[0].fname);
+			addMod2List(version, functArray);
+			break;
 		case eMOD_ANY: /* this is mostly to keep the compiler happy! */
 			DBGPRINTF("PROGRAM ERROR: eMOD_ANY set as module type\n");
 			assert(0);
@@ -885,6 +893,9 @@ static void modPrintList(void)
 		case eMOD_STRGEN:
 			dbgprintf("strgen");
 			break;
+		case eMOD_FUNCTION:
+			dbgprintf("function");
+			break;
 		case eMOD_ANY: /* this is mostly to keep the compiler happy! */
 			DBGPRINTF("PROGRAM ERROR: eMOD_ANY set as module type\n");
 			assert(0);
@@ -942,6 +953,10 @@ static void modPrintList(void)
 			dbgprintf("Strgen Module Entry Points\n");
 			dbgprintf("\tstrgen:            0x%lx\n", (unsigned long) pMod->mod.sm.strgen);
 			break;
+		case eMOD_FUNCTION:
+			dbgprintf("Function Module Entry Points\n");
+			dbgprintf("\tgetFunctArray:     0x%lx\n", (unsigned long) pMod->mod.fm.getFunctArray);
+			break;
 		case eMOD_ANY: /* this is mostly to keep the compiler happy! */
 			break;
 		}
@@ -1153,7 +1168,7 @@ Load(uchar *pModName, sbool bConfLoad, struct nvlst *lst)
 			if(pModInfo->setModCnf != NULL && localRet == RS_RET_OK) {
 				if(!strncmp((char*)pModName, "builtin:", sizeof("builtin:")-1)) {
 					if(pModInfo->bSetModCnfCalled) {
-						errmsg.LogError(0, RS_RET_DUP_PARAM,
+						LogError(0, RS_RET_DUP_PARAM,
 						    "parameters for built-in module %s already set - ignored\n",
 						    pModName);
 						ABORT_FINALIZE(RS_RET_DUP_PARAM);
@@ -1260,16 +1275,16 @@ Load(uchar *pModName, sbool bConfLoad, struct nvlst *lst)
 
 	if(!pModHdlr) {
 		if(iLoadCnt) {
-			errmsg.LogError(0, RS_RET_MODULE_LOAD_ERR_DLOPEN, "could not load module '%s', dlopen: %s\n",
+			LogError(0, RS_RET_MODULE_LOAD_ERR_DLOPEN, "could not load module '%s', dlopen: %s\n",
 					pPathBuf, dlerror());
 		} else {
-			errmsg.LogError(0, NO_ERRCODE, "could not load module '%s', ModDir was '%s'\n", pPathBuf,
+			LogError(0, NO_ERRCODE, "could not load module '%s', ModDir was '%s'\n", pPathBuf,
 			                               ((pModDir == NULL) ? _PATH_MODDIR : (char *)pModDir));
 		}
 		ABORT_FINALIZE(RS_RET_MODULE_LOAD_ERR_DLOPEN);
 	}
 	if(!(pModInit = dlsym(pModHdlr, "modInit"))) {
-		errmsg.LogError(0, RS_RET_MODULE_LOAD_ERR_NO_INIT,
+		LogError(0, RS_RET_MODULE_LOAD_ERR_NO_INIT,
 			 	"could not load module '%s', dlsym: %s\n", pPathBuf, dlerror());
 		dlclose(pModHdlr);
 		ABORT_FINALIZE(RS_RET_MODULE_LOAD_ERR_NO_INIT);
@@ -1281,7 +1296,7 @@ Load(uchar *pModName, sbool bConfLoad, struct nvlst *lst)
 #else
 	if((iRet = doModInit(pModInit, (uchar*) pModName, pModHdlr, &pModInfo)) != RS_RET_OK) {
 #endif
-		errmsg.LogError(0, RS_RET_MODULE_LOAD_ERR_INIT_FAILED,
+		LogError(0, RS_RET_MODULE_LOAD_ERR_INIT_FAILED,
 				"could not load module '%s', rsyslog error %d\n", pPathBuf, iRet);
 		dlclose(pModHdlr);
 		ABORT_FINALIZE(RS_RET_MODULE_LOAD_ERR_INIT_FAILED);
@@ -1293,7 +1308,7 @@ Load(uchar *pModName, sbool bConfLoad, struct nvlst *lst)
 			if(lst != NULL) {
 				localRet = pModInfo->setModCnf(lst);
 				if(localRet != RS_RET_OK) {
-					errmsg.LogError(0, localRet,
+					LogError(0, localRet,
 						"module '%s', failed processing config parameters",
 						pPathBuf);
 					ABORT_FINALIZE(localRet);
@@ -1333,7 +1348,7 @@ modulesProcessCnf(struct cnfobj *o)
 	cnfparamsPrint(&pblk, pvals);
 	typeIdx = cnfparamGetIdx(&pblk, "load");
 	if(pvals[typeIdx].bUsed == 0) {
-		errmsg.LogError(0, RS_RET_CONF_RQRD_PARAM_MISSING, "module type missing");
+		LogError(0, RS_RET_CONF_RQRD_PARAM_MISSING, "module type missing");
 		ABORT_FINALIZE(RS_RET_CONF_RQRD_PARAM_MISSING);
 	}
 
@@ -1437,7 +1452,6 @@ Release(const char *srcFile, modInfo_t **ppThis)
 BEGINObjClassExit(module, OBJ_IS_LOADABLE_MODULE) /* CHANGE class also in END MACRO! */
 CODESTARTObjClassExit(module)
 	/* release objects we no longer need */
-	objRelease(errmsg, CORE_COMPONENT);
 	free(pModDir);
 #	ifdef DEBUG
 	modUsrPrintAll(); /* debug aid - TODO: integrate with debug.c, at least the settings! */
@@ -1499,7 +1513,6 @@ BEGINAbstractObjClassInit(module, 1, OBJ_IS_CORE_MODULE) /* class, version - CHA
 	}
 
 	/* request objects we use */
-	CHKiRet(objUse(errmsg, CORE_COMPONENT));
 ENDObjClassInit(module)
 
 /* vi:set ai:
