@@ -1,7 +1,7 @@
 /* nsd_gtls.c
  *
  * An implementation of the nsd interface for GnuTLS.
- * 
+ *
  * Copyright (C) 2007-2016 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of the rsyslog runtime library.
@@ -64,7 +64,6 @@ MODULE_TYPE_KEEP
 
 /* static data */
 DEFobjStaticHelpers
-DEFobjCurrIf(errmsg)
 DEFobjCurrIf(glbl)
 DEFobjCurrIf(net)
 DEFobjCurrIf(datetime)
@@ -82,7 +81,7 @@ static pthread_mutex_t mutGtlsStrerror;
  */
 #define ABORTgnutls { \
 		uchar *pErr = gtlsStrerror(gnuRet); \
-		errmsg.LogError(0, RS_RET_GNUTLS_ERR, "unexpected GnuTLS error %d in %s:%d: %s\n", \
+		LogError(0, RS_RET_GNUTLS_ERR, "unexpected GnuTLS error %d in %s:%d: %s\n", \
 	gnuRet, __FILE__, __LINE__, pErr); \
 		free(pErr); \
 		ABORT_FINALIZE(RS_RET_GNUTLS_ERR); \
@@ -91,7 +90,7 @@ static pthread_mutex_t mutGtlsStrerror;
 #define CHKgnutls(x) { \
 	gnuRet = (x); \
 	if(gnuRet == GNUTLS_E_FILE_ERROR) { \
-		errmsg.LogError(0, RS_RET_GNUTLS_ERR, "error reading file - a common cause is that the " \
+		LogError(0, RS_RET_GNUTLS_ERR, "error reading file - a common cause is that the " \
 			"file  does not exist"); \
 		ABORT_FINALIZE(RS_RET_GNUTLS_ERR); \
 	} else if(gnuRet != 0) { \
@@ -132,25 +131,25 @@ readFile(uchar *pszFile, gnutls_datum_t *pBuf)
 	pBuf->data = NULL;
 
 	if((fd = open((char*)pszFile, O_RDONLY)) == -1) {
-		errmsg.LogError(errno, RS_RET_FILE_NOT_FOUND, "can not read file '%s'", pszFile);
+		LogError(errno, RS_RET_FILE_NOT_FOUND, "can not read file '%s'", pszFile);
 		ABORT_FINALIZE(RS_RET_FILE_NOT_FOUND);
 	}
 
 	if(fstat(fd, &stat_st) == -1) {
-		errmsg.LogError(errno, RS_RET_FILE_NO_STAT, "can not stat file '%s'", pszFile);
+		LogError(errno, RS_RET_FILE_NO_STAT, "can not stat file '%s'", pszFile);
 		ABORT_FINALIZE(RS_RET_FILE_NO_STAT);
 	}
 
 	/* 1MB limit */
 	if(stat_st.st_size > 1024 * 1024) {
-		errmsg.LogError(0, RS_RET_FILE_TOO_LARGE, "file '%s' too large, max 1MB", pszFile);
+		LogError(0, RS_RET_FILE_TOO_LARGE, "file '%s' too large, max 1MB", pszFile);
 		ABORT_FINALIZE(RS_RET_FILE_TOO_LARGE);
 	}
 
 	CHKmalloc(pBuf->data = MALLOC(stat_st.st_size));
 	pBuf->size = stat_st.st_size;
 	if(read(fd,  pBuf->data, stat_st.st_size) != stat_st.st_size) {
-		errmsg.LogError(0, RS_RET_IO_ERROR, "error or incomplete read of file '%s'", pszFile);
+		LogError(0, RS_RET_IO_ERROR, "error or incomplete read of file '%s'", pszFile);
 		ABORT_FINALIZE(RS_RET_IO_ERROR);
 	}
 
@@ -190,7 +189,7 @@ gtlsLoadOurCertKey(nsd_gtls_t *pThis)
 	keyFile = glbl.GetDfltNetstrmDrvrKeyFile();
 
 	if(certFile == NULL || keyFile == NULL) {
-		/* in this case, we can not set our certificate. If we are 
+		/* in this case, we can not set our certificate. If we are
 		 * a client and the server is running in "anon" auth mode, this
 		 * may be well acceptable. In other cases, we will see some
 		 * more error messages down the road. -- rgerhards, 2008-07-02
@@ -202,9 +201,13 @@ gtlsLoadOurCertKey(nsd_gtls_t *pThis)
 
 	/* try load certificate */
 	CHKiRet(readFile(certFile, &data));
-	CHKgnutls(gnutls_x509_crt_init(&pThis->ourCert));
+	pThis->nOurCerts = sizeof(pThis->pOurCerts) / sizeof(gnutls_x509_crt_t);
+	gnuRet = gnutls_x509_crt_list_import(pThis->pOurCerts, &pThis->nOurCerts,
+		&data, GNUTLS_X509_FMT_PEM,  GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED);
+	if(gnuRet < 0) {
+		ABORTgnutls;
+	}
 	pThis->bOurCertIsInit = 1;
-	CHKgnutls(gnutls_x509_crt_import(pThis->ourCert, &data, GNUTLS_X509_FMT_PEM));
 	free(data.data);
 	data.data = NULL;
 
@@ -220,7 +223,9 @@ finalize_it:
 		if(data.data != NULL)
 			free(data.data);
 		if(pThis->bOurCertIsInit) {
-			gnutls_x509_crt_deinit(pThis->ourCert);
+			for(unsigned i=0; i<pThis->nOurCerts; ++i) {
+				gnutls_x509_crt_deinit(pThis->pOurCerts[i]);
+			}
 			pThis->bOurCertIsInit = 0;
 		}
 		if(pThis->bOurKeyIsInit) {
@@ -265,8 +270,8 @@ gtlsClientCertCallback(gnutls_session_t session,
 #else
 	st->type = GNUTLS_CRT_X509;
 #endif
-	st->ncerts = 1;
-	st->cert.x509 = &pThis->ourCert;
+	st->ncerts = pThis->nOurCerts;
+	st->cert.x509 = pThis->pOurCerts;
 	st->key.x509 = pThis->ourKey;
 	st->deinit_all = 0;
 
@@ -562,7 +567,7 @@ finalize_it:
 static rsRetVal
 gtlsAddOurCert(void)
 {
-	int gnuRet;
+	int gnuRet = 0;
 	uchar *keyFile;
 	uchar *certFile;
 	uchar *pGnuErr; /* for GnuTLS error reporting */
@@ -573,12 +578,12 @@ gtlsAddOurCert(void)
 	dbgprintf("GTLS certificate file: '%s'\n", certFile);
 	dbgprintf("GTLS key file: '%s'\n", keyFile);
 	if(certFile == NULL) {
-		errmsg.LogError(0, RS_RET_CERT_MISSING, "error: certificate file is not set, cannot "
+		LogError(0, RS_RET_CERT_MISSING, "error: certificate file is not set, cannot "
 				"continue");
 		ABORT_FINALIZE(RS_RET_CERT_MISSING);
 	}
 	if(keyFile == NULL) {
-		errmsg.LogError(0, RS_RET_CERTKEY_MISSING, "error: key file is not set, cannot "
+		LogError(0, RS_RET_CERTKEY_MISSING, "error: key file is not set, cannot "
 				"continue");
 		ABORT_FINALIZE(RS_RET_CERTKEY_MISSING);
 	}
@@ -588,7 +593,7 @@ finalize_it:
 	if(iRet != RS_RET_OK && iRet != RS_RET_CERT_MISSING && iRet != RS_RET_CERTKEY_MISSING) {
 		pGnuErr = gtlsStrerror(gnuRet);
 		errno = 0;
-		errmsg.LogError(0, iRet, "error adding our certificate. GnuTLS error %d, message: '%s', "
+		LogError(0, iRet, "error adding our certificate. GnuTLS error %d, message: '%s', "
 				"key: '%s', cert: '%s'", gnuRet, pGnuErr, keyFile, certFile);
 		free(pGnuErr);
 	}
@@ -609,28 +614,28 @@ gtlsGlblInit(void)
 	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
 	#endif
 	CHKgnutls(gnutls_global_init());
-	
+
 	/* X509 stuff */
 	CHKgnutls(gnutls_certificate_allocate_credentials(&xcred));
 
 	/* sets the trusted cas file */
 	cafile = glbl.GetDfltNetstrmDrvrCAF();
 	if(cafile == NULL) {
-		errmsg.LogError(0, RS_RET_CA_CERT_MISSING, "error: ca certificate is not set, cannot "
+		LogError(0, RS_RET_CA_CERT_MISSING, "error: ca certificate is not set, cannot "
 				"continue");
 		ABORT_FINALIZE(RS_RET_CA_CERT_MISSING);
 	}
 	dbgprintf("GTLS CA file: '%s'\n", cafile);
 	gnuRet = gnutls_certificate_set_x509_trust_file(xcred, (char*)cafile, GNUTLS_X509_FMT_PEM);
 	if(gnuRet == GNUTLS_E_FILE_ERROR) {
-		errmsg.LogError(0, RS_RET_GNUTLS_ERR,
+		LogError(0, RS_RET_GNUTLS_ERR,
 			"error reading certificate file '%s' - a common cause is that the "
 			"file  does not exist", cafile);
 		ABORT_FINALIZE(RS_RET_GNUTLS_ERR);
 	} else if(gnuRet < 0) {
 		/* TODO; a more generic error-tracking function (this one based on CHKgnutls()) */
 		uchar *pErr = gtlsStrerror(gnuRet);
-		errmsg.LogError(0, RS_RET_GNUTLS_ERR, "unexpected GnuTLS error %d in %s:%d: %s\n",
+		LogError(0, RS_RET_GNUTLS_ERR, "unexpected GnuTLS error %d in %s:%d: %s\n",
 		gnuRet, __FILE__, __LINE__, pErr);
 		free(pErr);
 		ABORT_FINALIZE(RS_RET_GNUTLS_ERR);
@@ -638,7 +643,7 @@ gtlsGlblInit(void)
 
 	if(GetGnuTLSLoglevel() > 0){
 		gnutls_global_set_log_function(logFunction);
-		gnutls_global_set_log_level(GetGnuTLSLoglevel()); 
+		gnutls_global_set_log_level(GetGnuTLSLoglevel());
 		/* 0 (no) to 9 (most), 10 everything */
 	}
 
@@ -666,7 +671,7 @@ gtlsInitSession(nsd_gtls_t *pThis)
 
 	pThis->sess = session;
 
-#	if HAVE_GNUTLS_CERTIFICATE_SET_RETRIEVE_FUNCTION 
+#	if HAVE_GNUTLS_CERTIFICATE_SET_RETRIEVE_FUNCTION
 	/* store a pointer to ourselfs (needed by callback) */
 	gnutls_session_set_ptr(pThis->sess, (void*)pThis);
 	iRet = gtlsLoadOurCertKey(pThis); /* first load .pem files */
@@ -707,7 +712,7 @@ finalize_it:
 
 
 /* Obtain the CN from the DN field and hand it back to the caller
- * (which is responsible for destructing it). We try to follow 
+ * (which is responsible for destructing it). We try to follow
  * RFC2253 as far as it makes sense for our use-case. This function
  * is considered a compromise providing good-enough correctness while
  * limiting code size and complexity. If a problem occurs, we may enhance
@@ -824,7 +829,7 @@ gtlsChkPeerFingerprint(nsd_gtls_t *pThis, gnutls_x509_crt_t *pCert)
 		dbgprintf("invalid peer fingerprint, not permitted to talk to it\n");
 		if(pThis->bReportAuthErr == 1) {
 			errno = 0;
-			errmsg.LogError(0, RS_RET_INVALID_FINGERPRINT, "error: peer fingerprint '%s' unknown - we are "
+			LogError(0, RS_RET_INVALID_FINGERPRINT, "error: peer fingerprint '%s' unknown - we are "
 					"not permitted to talk to it", cstrGetSzStrNoNULL(pstrFingerprint));
 			pThis->bReportAuthErr = 0;
 		}
@@ -932,7 +937,7 @@ gtlsChkPeerName(nsd_gtls_t *pThis, gnutls_x509_crt_t *pCert)
 		if(pThis->bReportAuthErr == 1) {
 			cstrFinalize(pStr);
 			errno = 0;
-			errmsg.LogError(0, RS_RET_INVALID_FINGERPRINT, "error: peer name not authorized -  "
+			LogError(0, RS_RET_INVALID_FINGERPRINT, "error: peer name not authorized -  "
 					"not permitted to talk to it. Names: %s",
 					cstrGetSzStrNoNULL(pStr));
 			pThis->bReportAuthErr = 0;
@@ -975,14 +980,14 @@ gtlsChkPeerID(nsd_gtls_t *pThis)
 	if(list_size < 1) {
 		if(pThis->bReportAuthErr == 1) {
 			errno = 0;
-			errmsg.LogError(0, RS_RET_TLS_NO_CERT, "error: peer did not provide a certificate, "
+			LogError(0, RS_RET_TLS_NO_CERT, "error: peer did not provide a certificate, "
 					"not permitted to talk to it");
 			pThis->bReportAuthErr = 0;
 		}
 		ABORT_FINALIZE(RS_RET_TLS_NO_CERT);
 	}
 
-	/* If we reach this point, we have at least one valid certificate. 
+	/* If we reach this point, we have at least one valid certificate.
 	 * We always use only the first certificate. As of GnuTLS documentation, the
 	 * first certificate always contains the remote peer's own certificate. All other
 	 * certificates are issuer's certificates (up the chain). We are only interested
@@ -1032,7 +1037,7 @@ gtlsChkPeerCertValidity(nsd_gtls_t *pThis)
 	cert_list = gnutls_certificate_get_peers(pThis->sess, &cert_list_size);
 	if(cert_list_size < 1) {
 		errno = 0;
-		errmsg.LogError(0, RS_RET_TLS_NO_CERT,
+		LogError(0, RS_RET_TLS_NO_CERT,
 			"peer did not provide a certificate, not permitted to talk to it");
 		ABORT_FINALIZE(RS_RET_TLS_NO_CERT);
 	}
@@ -1054,10 +1059,10 @@ gtlsChkPeerCertValidity(nsd_gtls_t *pThis)
 			dbgprintf("GnuTLS returned no specific reason for GNUTLS_CERT_INVALID, certificate "
 				 "status is %d\n", stateCert);
 		}
-		errmsg.LogError(0, NO_ERRCODE, "not permitted to talk to peer, certificate invalid: %s",
+		LogError(0, NO_ERRCODE, "not permitted to talk to peer, certificate invalid: %s",
 				pszErrCause);
 		gtlsGetCertInfo(pThis, &pStr);
-		errmsg.LogError(0, NO_ERRCODE, "invalid cert info: %s", cstrGetSzStrNoNULL(pStr));
+		LogError(0, NO_ERRCODE, "invalid cert info: %s", cstrGetSzStrNoNULL(pStr));
 		cstrDestruct(&pStr);
 		ABORT_FINALIZE(RS_RET_CERT_INVALID);
 	}
@@ -1077,10 +1082,10 @@ gtlsChkPeerCertValidity(nsd_gtls_t *pThis)
 		if(ttCert == -1)
 			ABORT_FINALIZE(RS_RET_TLS_CERT_ERR);
 		else if(ttCert > ttNow) {
-			errmsg.LogError(0, RS_RET_CERT_NOT_YET_ACTIVE, "not permitted to talk to peer: "
+			LogError(0, RS_RET_CERT_NOT_YET_ACTIVE, "not permitted to talk to peer: "
 					"certificate %d not yet active", i);
 			gtlsGetCertInfo(pThis, &pStr);
-			errmsg.LogError(0, RS_RET_CERT_NOT_YET_ACTIVE,
+			LogError(0, RS_RET_CERT_NOT_YET_ACTIVE,
 				"invalid cert info: %s", cstrGetSzStrNoNULL(pStr));
 			cstrDestruct(&pStr);
 			ABORT_FINALIZE(RS_RET_CERT_NOT_YET_ACTIVE);
@@ -1090,10 +1095,10 @@ gtlsChkPeerCertValidity(nsd_gtls_t *pThis)
 		if(ttCert == -1)
 			ABORT_FINALIZE(RS_RET_TLS_CERT_ERR);
 		else if(ttCert < ttNow) {
-			errmsg.LogError(0, RS_RET_CERT_EXPIRED, "not permitted to talk to peer: certificate"
+			LogError(0, RS_RET_CERT_EXPIRED, "not permitted to talk to peer: certificate"
 				" %d expired", i);
 			gtlsGetCertInfo(pThis, &pStr);
-			errmsg.LogError(0, RS_RET_CERT_EXPIRED, "invalid cert info: %s", cstrGetSzStrNoNULL(pStr));
+			LogError(0, RS_RET_CERT_EXPIRED, "invalid cert info: %s", cstrGetSzStrNoNULL(pStr));
 			cstrDestruct(&pStr);
 			ABORT_FINALIZE(RS_RET_CERT_EXPIRED);
 		}
@@ -1220,7 +1225,9 @@ CODESTARTobjDestruct(nsd_gtls)
 	}
 
 	if(pThis->bOurCertIsInit)
-		gnutls_x509_crt_deinit(pThis->ourCert);
+                  for(unsigned i=0; i<pThis->nOurCerts; ++i) {
+			gnutls_x509_crt_deinit(pThis->pOurCerts[i]);
+                  }
 	if(pThis->bOurKeyIsInit)
 		gnutls_x509_privkey_deinit(pThis->ourKey);
 	if(pThis->bHaveSess)
@@ -1241,7 +1248,7 @@ SetMode(nsd_t *pNsd, int mode)
 
 	ISOBJ_TYPE_assert((pThis), nsd_gtls);
 	if(mode != 0 && mode != 1) {
-		errmsg.LogError(0, RS_RET_INVALID_DRVR_MODE, "error: driver mode %d not supported by "
+		LogError(0, RS_RET_INVALID_DRVR_MODE, "error: driver mode %d not supported by "
 				"gtls netstream driver", mode);
 		ABORT_FINALIZE(RS_RET_INVALID_DRVR_MODE);
 	}
@@ -1276,7 +1283,7 @@ SetAuthMode(nsd_t *pNsd, uchar *mode)
 	} else if(!strcasecmp((char*) mode, "anon")) {
 		pThis->authMode = GTLS_AUTH_CERTANON;
 	} else {
-		errmsg.LogError(0, RS_RET_VALUE_NOT_SUPPORTED, "error: authentication mode '%s' not supported by "
+		LogError(0, RS_RET_VALUE_NOT_SUPPORTED, "error: authentication mode '%s' not supported by "
 				"gtls netstream driver", mode);
 		ABORT_FINALIZE(RS_RET_VALUE_NOT_SUPPORTED);
 	}
@@ -1288,7 +1295,7 @@ finalize_it:
 }
 
 
-/* Set permitted peers. It is depending on the auth mode if this are 
+/* Set permitted peers. It is depending on the auth mode if this are
  * fingerprints or names. -- rgerhards, 2008-05-19
  */
 static rsRetVal
@@ -1302,7 +1309,7 @@ SetPermPeers(nsd_t *pNsd, permittedPeers_t *pPermPeers)
 		FINALIZE;
 
 	if(pThis->authMode != GTLS_AUTH_CERTFINGERPRINT && pThis->authMode != GTLS_AUTH_CERTNAME) {
-		errmsg.LogError(0, RS_RET_VALUE_NOT_IN_THIS_MODE, "authentication not supported by "
+		LogError(0, RS_RET_VALUE_NOT_IN_THIS_MODE, "authentication not supported by "
 			"gtls netstream driver in the configured authentication mode - ignored");
 		ABORT_FINALIZE(RS_RET_VALUE_NOT_IN_THIS_MODE);
 	}
@@ -1508,7 +1515,7 @@ AcceptConnReq(nsd_t *pNsd, nsd_t **ppNew)
 	CHKiRet(nsd_gtlsConstruct(&pNew)); // TODO: prevent construct/destruct!
 	CHKiRet(nsd_ptcp.Destruct(&pNew->pTcp));
 	CHKiRet(nsd_ptcp.AcceptConnReq(pThis->pTcp, &pNew->pTcp));
-	
+
 	if(pThis->iMode == 0) {
 		/* we are in non-TLS mode, so we are done */
 		*ppNew = (nsd_t*) pNew;
@@ -1526,7 +1533,7 @@ AcceptConnReq(nsd_t *pNsd, nsd_t **ppNew)
 		if(gnutls_priority_set_direct(pNew->sess,
 					(const char*) pNew->gnutlsPriorityString,
 					&error_position)==GNUTLS_E_INVALID_REQUEST) {
-			errmsg.LogError(0, RS_RET_GNUTLS_ERR, "Syntax Error in"
+			LogError(0, RS_RET_GNUTLS_ERR, "Syntax Error in"
 					" Priority String: \"%s\"\n", error_position);
 		}
 	} else {
@@ -1534,7 +1541,7 @@ AcceptConnReq(nsd_t *pNsd, nsd_t **ppNew)
 		CHKgnutls(gnutls_set_default_priority(pNew->sess));
 	}
 
-	/* we now do the handshake. This is a bit complicated, because we are 
+	/* we now do the handshake. This is a bit complicated, because we are
 	 * on non-blocking sockets. Usually, the handshake will not complete
 	 * immediately, so that we need to retry it some time later.
 	 */
@@ -1548,7 +1555,7 @@ AcceptConnReq(nsd_t *pNsd, nsd_t **ppNew)
 		CHKiRet(gtlsChkPeerAuth(pNew));
 	} else {
 		uchar *pGnuErr = gtlsStrerror(gnuRet);
-		errmsg.LogError(0, RS_RET_TLS_HANDSHAKE_ERR, 
+		LogError(0, RS_RET_TLS_HANDSHAKE_ERR,
 			"gnutls returned error on handshake: %s\n", pGnuErr);
 		free(pGnuErr);
 		ABORT_FINALIZE(RS_RET_TLS_HANDSHAKE_ERR);
@@ -1580,7 +1587,7 @@ finalize_it:
  * implementation, it is on the stack and extremely likely to change). To
  * work-around this problem, we allocate a buffer ourselfs and always receive
  * into that buffer. We pass data on to the caller only after we have received it.
- * To save some space, we allocate that internal buffer only when it is actually 
+ * To save some space, we allocate that internal buffer only when it is actually
  * needed, which means when we reach this function for the first time. To keep
  * the algorithm simple, we always supply data only from the internal buffer,
  * even if it is a single byte. As we have a stream, the caller must be prepared
@@ -1607,7 +1614,7 @@ Rcv(nsd_t *pNsd, uchar *pBuf, ssize_t *pLenBuf, int *const oserr)
 
 	/* --- in TLS mode now --- */
 
-	/* Buffer logic applies only if we are in TLS mode. Here we 
+	/* Buffer logic applies only if we are in TLS mode. Here we
 	 * assume that we will switch from plain to TLS, but never back. This
 	 * assumption may be unsafe, but it is the model for the time being and I
 	 * do not see any valid reason why we should switch back to plain TCP after
@@ -1646,7 +1653,7 @@ Rcv(nsd_t *pNsd, uchar *pBuf, ssize_t *pLenBuf, int *const oserr)
 	*pLenBuf = iBytesCopy;
 
 finalize_it:
-	if (iRet != RS_RET_OK && 
+	if (iRet != RS_RET_OK &&
 		iRet != RS_RET_RETRY) {
 		/* We need to free the receive buffer in error error case unless a retry is wanted. , if we
 		 * allocated one. -- rgerhards, 2008-12-03 -- moved here by alorbach, 2015-12-01
@@ -1692,7 +1699,7 @@ Send(nsd_t *pNsd, uchar *pBuf, ssize_t *pLenBuf)
 		}
 		if(iSent != GNUTLS_E_INTERRUPTED && iSent != GNUTLS_E_AGAIN) {
 			uchar *pErr = gtlsStrerror(iSent);
-			errmsg.LogError(0, RS_RET_GNUTLS_ERR, "unexpected GnuTLS error %d - this "
+			LogError(0, RS_RET_GNUTLS_ERR, "unexpected GnuTLS error %d - this "
 				"could be caused by a broken connection. GnuTLS reports: %s \n",
 				iSent, pErr);
 			free(pErr);
@@ -1768,7 +1775,7 @@ Connect(nsd_t *pNsd, int family, uchar *port, uchar *host, char *device)
 
 	if(pThis->iMode == 0)
 		FINALIZE;
-	
+
 	/* we reach this point if in TLS mode */
 	CHKgnutls(gnutls_init(&pThis->sess, GNUTLS_CLIENT));
 	pThis->bHaveSess = 1;
@@ -1784,7 +1791,7 @@ Connect(nsd_t *pNsd, int family, uchar *port, uchar *host, char *device)
 	gnutls_session_set_ptr(pThis->sess, (void*)pThis);
 	iRet = gtlsLoadOurCertKey(pThis); /* first load .pem files */
 	if(iRet == RS_RET_OK) {
-#		if HAVE_GNUTLS_CERTIFICATE_SET_RETRIEVE_FUNCTION 
+#		if HAVE_GNUTLS_CERTIFICATE_SET_RETRIEVE_FUNCTION
 		gnutls_certificate_set_retrieve_function(xcred, gtlsClientCertCallback);
 #		else
 		gnutls_certificate_client_set_retrieve_function(xcred, gtlsClientCertCallback);
@@ -1798,7 +1805,7 @@ Connect(nsd_t *pNsd, int family, uchar *port, uchar *host, char *device)
 		if(gnutls_priority_set_direct(pThis->sess,
 					(const char*) pThis->gnutlsPriorityString,
 					&error_position)==GNUTLS_E_INVALID_REQUEST) {
-			errmsg.LogError(0, RS_RET_GNUTLS_ERR, "Syntax Error in"
+			LogError(0, RS_RET_GNUTLS_ERR, "Syntax Error in"
 					" Priority String: \"%s\"\n", error_position);
 		}
 	} else {
@@ -1838,8 +1845,8 @@ Connect(nsd_t *pNsd, int family, uchar *port, uchar *host, char *device)
 	CHKgnutls(gnutls_handshake(pThis->sess));
 	dbgprintf("GnuTLS handshake succeeded\n");
 
-	/* now check if the remote peer is permitted to talk to us - ideally, we 
-	 * should do this during the handshake, but GnuTLS does not yet provide 
+	/* now check if the remote peer is permitted to talk to us - ideally, we
+	 * should do this during the handshake, but GnuTLS does not yet provide
 	 * the necessary callbacks -- rgerhards, 2008-05-26
 	 */
 	CHKiRet(gtlsChkPeerAuth(pThis));
@@ -1905,7 +1912,6 @@ CODESTARTObjClassExit(nsd_gtls)
 	objRelease(net, LM_NET_FILENAME);
 	objRelease(glbl, CORE_COMPONENT);
 	objRelease(datetime, CORE_COMPONENT);
-	objRelease(errmsg, CORE_COMPONENT);
 ENDObjClassExit(nsd_gtls)
 
 
@@ -1915,7 +1921,6 @@ ENDObjClassExit(nsd_gtls)
  */
 BEGINObjClassInit(nsd_gtls, 1, OBJ_IS_LOADABLE_MODULE) /* class, version */
 	/* request objects we use */
-	CHKiRet(objUse(errmsg, CORE_COMPONENT));
 	CHKiRet(objUse(datetime, CORE_COMPONENT));
 	CHKiRet(objUse(glbl, CORE_COMPONENT));
 	CHKiRet(objUse(net, LM_NET_FILENAME));
