@@ -1,7 +1,7 @@
 /* This is the template processing code of rsyslog.
  * begun 2004-11-17 rgerhards
  *
- * Copyright 2004-2017 Rainer Gerhards and Adiscon
+ * Copyright 2004-2018 Rainer Gerhards and Adiscon
  *
  * This file is part of rsyslog.
  *
@@ -65,6 +65,7 @@ static struct cnfparamdescr cnfparamdescr[] = {
 	{ "option.stdsql", eCmdHdlrBinary, 0 },
 	{ "option.sql", eCmdHdlrBinary, 0 },
 	{ "option.json", eCmdHdlrBinary, 0 },
+	{ "option.jsonf", eCmdHdlrBinary, 0 },
 	{ "option.casesensitive", eCmdHdlrBinary, 0 }
 };
 static struct cnfparamblk pblk =
@@ -106,7 +107,8 @@ static struct cnfparamblk pblkProperty =
 
 static struct cnfparamdescr cnfparamdescrConstant[] = {
 	{ "value", eCmdHdlrString, 1 },
-	{ "outname", eCmdHdlrString, 0 },
+	{ "format", eCmdHdlrString, 0 },
+	{ "outname", eCmdHdlrString, 0 }
 };
 static struct cnfparamblk pblkConstant =
 	{ CNFPARAMBLK_VERSION,
@@ -189,6 +191,13 @@ tplToString(struct template *__restrict__ const pTpl,
 	 */
 	pTpe = pTpl->pEntryRoot;
 	iBuf = 0;
+	const int extra_space = (pTpl->optFormatEscape == JSONF) ? 1 : 3;
+	if(pTpl->optFormatEscape == JSONF) {
+		if(iparam->lenBuf < 2) /* we reserve one char for the final \0! */
+			CHKiRet(ExtendBuf(iparam, 2));
+		iBuf = 1;
+		*iparam->param = '{';
+	}
 	while(pTpe != NULL) {
 		if(pTpe->eEntryType == CONSTANT) {
 			pVal = (uchar*) pTpe->data.constant.pConstant;
@@ -218,11 +227,16 @@ tplToString(struct template *__restrict__ const pTpl,
 		/* got source, now copy over */
 		if(iLenVal > 0) { /* may be zero depending on property */
 			/* first, make sure buffer fits */
-			if(iBuf + iLenVal >= iparam->lenBuf) /* we reserve one char for the final \0! */
+			if(iBuf + iLenVal + extra_space >= iparam->lenBuf) /* we reserve one char for the final \0! */
 				CHKiRet(ExtendBuf(iparam, iBuf + iLenVal + 1));
 
 			memcpy(iparam->param + iBuf, pVal, iLenVal);
 			iBuf += iLenVal;
+			if(pTpl->optFormatEscape == JSONF) {
+				memcpy(iparam->param + iBuf,
+					(pTpe->pNext == NULL) ? "}\n" : ", ", 2);
+				iBuf += 2;
+			}
 		}
 
 		if(bMustBeFreed) {
@@ -1430,7 +1444,10 @@ createConstantTpe(struct template *pTpl, struct cnfobj *o)
 	struct templateEntry *pTpe;
 	es_str_t *value = NULL; /* init just to keep compiler happy - mandatory parameter */
 	int i;
+	int is_jsonf = 0;
 	struct cnfparamvals *pvals = NULL;
+	struct json_object *json = NULL;
+	struct json_object *jval = NULL;
 	uchar *outname = NULL;
 	DEFiRet;
 
@@ -1449,20 +1466,31 @@ createConstantTpe(struct template *pTpl, struct cnfobj *o)
 			value = pvals[i].val.d.estr;
 		} else if(!strcmp(pblkConstant.descr[i].name, "outname")) {
 			outname = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
+		} else if(!strcmp(pblkConstant.descr[i].name, "format")) {
+			if(!es_strbufcmp(pvals[i].val.d.estr, (uchar*)"jsonf", sizeof("jsonf")-1)) {
+				is_jsonf = 1;
+			} else {
+				uchar *typeStr = (uchar*) es_str2cstr(pvals[i].val.d.estr, NULL);
+				LogError(0, RS_RET_ERR, "invalid format type '%s' for constant",
+					typeStr);
+				free(typeStr);
+				ABORT_FINALIZE(RS_RET_ERR);
+			}
 		} else {
-			dbgprintf("template:constantTpe: program error, non-handled "
-			  "param '%s'\n", pblkConstant.descr[i].name);
+			LogError(0, RS_RET_INTERNAL_ERROR,
+				"template:constantTpe: program error, non-handled "
+				"param '%s'\n", pblkConstant.descr[i].name);
 		}
 	}
 
-	/* sanity check */
-	if(value == NULL) {
-		LogError(0, RS_RET_INTERNAL_ERROR, "createConstantTpe(): "
-			"internal error, variable 'value'==NULL, which is "
-			"not permitted.");
-		ABORT_FINALIZE(RS_RET_INTERNAL_ERROR);
+	if(is_jsonf && outname == NULL) {
+		parser_errmsg("constant set to format jsonf, but outname not specified - aborting");
+		ABORT_FINALIZE(RS_RET_ERR);
 	}
 
+	/* just double-check */
+	assert(value != NULL);
+
 	/* apply */
 	CHKmalloc(pTpe = tpeConstruct(pTpl));
 	es_unescapeStr(value);
@@ -1470,8 +1498,22 @@ createConstantTpe(struct template *pTpl, struct cnfobj *o)
 	pTpe->fieldName = outname;
 	if(outname != NULL)
 		pTpe->lenFieldName = ustrlen(outname);
-	pTpe->data.constant.iLenConstant = es_strlen(value);
-	pTpe->data.constant.pConstant = (uchar*)es_str2cstr(value, NULL);
+	if(is_jsonf) {
+		CHKmalloc(json = json_object_new_object());
+		const char *sz = es_str2cstr(value, NULL);
+		CHKmalloc(sz);
+		CHKmalloc(jval = json_object_new_string(sz));
+		free((void*)sz);
+		json_object_object_add(json, (char*)outname, jval);
+		CHKmalloc(sz = json_object_get_string(json));
+		const size_t len_json = strlen(sz) - 4;
+		CHKmalloc(pTpe->data.constant.pConstant = (uchar*) strndup(sz+2, len_json));
+		pTpe->data.constant.iLenConstant = ustrlen(pTpe->data.constant.pConstant);
+		json_object_put(json);
+	} else {
+		pTpe->data.constant.iLenConstant = es_strlen(value);
+		pTpe->data.constant.pConstant = (uchar*)es_str2cstr(value, NULL);
+	}
 
 finalize_it:
 	if(pvals != NULL)
@@ -1897,7 +1939,7 @@ tplProcessCnf(struct cnfobj *o)
 	enum { T_STRING, T_PLUGIN, T_LIST, T_SUBTREE }
 		tplType = T_STRING; /* init just to keep compiler happy: mandatory parameter */
 	int i;
-	int o_sql=0, o_stdsql=0, o_json=0, o_casesensitive=0; /* options */
+	int o_sql=0, o_stdsql=0, o_jsonf=0, o_json=0, o_casesensitive=0; /* options */
 	int numopts;
 	rsRetVal localRet;
 	DEFiRet;
@@ -1958,6 +2000,8 @@ tplProcessCnf(struct cnfobj *o)
 			o_sql = pvals[i].val.d.n;
 		} else if(!strcmp(pblk.descr[i].name, "option.json")) {
 			o_json = pvals[i].val.d.n;
+		} else if(!strcmp(pblk.descr[i].name, "option.jsonf")) {
+			o_jsonf = pvals[i].val.d.n;
 		} else if(!strcmp(pblk.descr[i].name, "option.casesensitive")) {
 			o_casesensitive = pvals[i].val.d.n;
 		} else {
@@ -2032,6 +2076,7 @@ tplProcessCnf(struct cnfobj *o)
 	if(o_sql) ++numopts;
 	if(o_stdsql) ++numopts;
 	if(o_json) ++numopts;
+	if(o_jsonf) ++numopts;
 	if(numopts > 1) {
 		LogError(0, RS_RET_ERR, "template '%s' has multiple incompatible "
 			"options of sql, stdsql or json specified", name);
@@ -2084,6 +2129,8 @@ tplProcessCnf(struct cnfobj *o)
 		pTpl->optFormatEscape = SQL_ESCAPE;
 	else if(o_json)
 		pTpl->optFormatEscape = JSON_ESCAPE;
+	else if(o_jsonf)
+		pTpl->optFormatEscape = JSONF;
 
 	if(o_casesensitive)
 		pTpl->optCaseSensitive = 1;
@@ -2268,6 +2315,8 @@ void tplPrintList(rsconf_t *conf)
 			dbgprintf("[JSON-Escaped Format] ");
 		else if(pTpl->optFormatEscape == STDSQL_ESCAPE)
 			dbgprintf("[SQL-Format (standard SQL)] ");
+		else if(pTpl->optFormatEscape == JSONF)
+			dbgprintf("[JSON fields] ");
 		if(pTpl->optCaseSensitive)
 			dbgprintf("[Case Sensitive Vars] ");
 		dbgprintf("\n");
