@@ -1,12 +1,6 @@
 #!/bin/bash
 # addd 2016-03-22 by RGerhards, released under ASL 2.0
 
-uname
-if [ `uname` = "FreeBSD" ] ; then
-   echo "This test currently does not work on FreeBSD."
-   exit 77
-fi
-
 . $srcdir/diag.sh init
 . $srcdir/diag.sh generate-conf
 . $srcdir/diag.sh add-conf '
@@ -30,7 +24,7 @@ rm tmp.in
 #. $srcdir/diag.sh tcpflood -m1 -M"\"<165>1 2003-08-24T05:14:15.000003-07:00 192.0.2.1 tcpflood 8710 - - msgnum:0000000 test   test     test\""
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
-echo "msgnum:0000000 test test test" | cmp rsyslog.out.log
+echo "msgnum:0000000 test test test" | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "invalid message recorded, rsyslog.out.log is:"
   cat rsyslog.out.log
