@@ -42,6 +42,8 @@
 #export RSYSLOG_DEBUGLOG="log"
 TB_TIMEOUT_STARTSTOP=400 # timeout for start/stop rsyslogd in tenths (!) of a second 400 => 40 sec
 # note that 40sec for the startup should be sufficient even on very slow machines. we changed this from 2min on 2017-12-12
+export RSYSLOG_DEBUG_TIMEOUTS_TO_STDERR="on"  # we want to know when we loose messages due to timeouts
+
 
 function rsyslog_testbench_test_url_access() {
     local missing_requirements=
@@ -69,8 +71,8 @@ dep_zk_url=http://www-us.apache.org/dist/zookeeper/zookeeper-3.4.10/zookeeper-3.
 dep_zk_cached_file=$dep_cache_dir/zookeeper-3.4.10.tar.gz
 dep_kafka_url=http://www-us.apache.org/dist/kafka/0.10.2.1/kafka_2.12-0.10.2.1.tgz
 if [ -z "$ES_DOWNLOAD" ]; then
-	dep_es_url=https://artifacts.elastic.co/downloads/elasticsearch/elasticsearch-5.6.3.tar.gz
-	dep_es_cached_file=$dep_cache_dir/elasticsearch-5.6.3.tar.gz
+	dep_es_url=https://artifacts.elastic.co/downloads/elasticsearch/elasticsearch-5.6.9.tar.gz
+	dep_es_cached_file=$dep_cache_dir/elasticsearch-5.6.9.tar.gz
 else
 	dep_es_url="https://artifacts.elastic.co/downloads/elasticsearch/$ES_DOWNLOAD"
 	dep_es_cached_file="$dep_cache_dir/$ES_DOWNLOAD"
@@ -122,25 +124,31 @@ case $1 in
 		echo "------------------------------------------------------------"
 		echo "Test: $0"
 		echo "------------------------------------------------------------"
+		# we assume TZ is set, else most test will fail. So let's ensure
+		# this really is the case
+		if [ -z $TZ ]; then
+			echo "testbench: TZ env var not set, setting it to UTC"
+			export TZ=UTC
+		fi
+
 		cp $srcdir/testsuites/diag-common.conf diag-common.conf
 		cp $srcdir/testsuites/diag-common2.conf diag-common2.conf
 		rm -f rsyslogd.started work-*.conf rsyslog.random.data
 		rm -f rsyslogd2.started work-*.conf
 		rm -f log log* # RSyslog debug output 
-		rm -f work rsyslog.out.log rsyslog2.out.log rsyslog.out.log.save # common work files
+		rm -f work rsyslog.out.* rsyslog2.out.log # common work files
 		rm -rf test-spool test-logdir stat-file1
-		rm -f rsyslog.out.*.log work-presort rsyslog.pipe
+		rm -f work-presort rsyslog.pipe
 		rm -f -r rsyslog.input.*
 		rm -f rsyslog.input rsyslog.empty rsyslog.input.* imfile-state* omkafka-failed.data
 		rm -f testconf.conf HOSTNAME
-		rm -f rsyslog.errorfile tmp.qi
-		rm -f core.* vghttp://www.rsyslog.com/testbench/echo-get.phpcore.* core*
+		rm -f rsyslog.errorfile tmp.qi nocert
+		rm -f core.* vgcore.* core*
 		# Note: rsyslog.action.*.include must NOT be deleted, as it
 		# is used to setup some parameters BEFORE calling init. This
 		# happens in chained test scripts. Delete on exit is fine,
 		# though.
 		mkdir test-spool
-		ulimit -c 4000000000
 		# note: TCPFLOOD_EXTRA_OPTS MUST NOT be unset in init, because
 		# some tests need to set it BEFORE calling init to accomodate
 		# their generic test drivers.
@@ -158,7 +166,6 @@ case $1 in
    'exit')	# cleanup
 		# detect any left-over hanging instance
 		nhanging=0
-		#for pid in $(ps -eo pid,cmd|grep '/tools/[r]syslogd' |sed -e 's/\( *\)\([0-9]*\).*/\2/');
 		for pid in $(ps -eo pid,args|grep '/tools/[r]syslogd' |sed -e 's/\( *\)\([0-9]*\).*/\2/');
 		do
 			echo "ERROR: left-over instance $pid, killing it"
@@ -174,13 +181,13 @@ case $1 in
 		# now real cleanup
 		rm -f rsyslogd.started work-*.conf diag-common.conf
    		rm -f rsyslogd2.started diag-common2.conf rsyslog.action.*.include
-		rm -f work rsyslog.out.log rsyslog2.out.log rsyslog.out.log.save # common work files
+		rm -f work rsyslog.out.* rsyslog2.out.log # common work files
 		rm -rf test-spool test-logdir stat-file1
-		rm -f rsyslog.out.*.log rsyslog.random.data work-presort rsyslog.pipe
+		rm -f rsyslog.random.data work-presort rsyslog.pipe
 		rm -f -r rsyslog.input.*
 		rm -f rsyslog.input rsyslog.conf.tlscert stat-file1 rsyslog.empty rsyslog.input.* imfile-state*
 		rm -f testconf.conf
-		rm -f rsyslog.errorfile tmp.qi
+		rm -f rsyslog.errorfile tmp.qi nocert
 		rm -f HOSTNAME imfile-state:.-rsyslog.input
 		unset TCPFLOOD_EXTRA_OPTS
 		echo  -------------------------------------------------------------------------------
@@ -188,9 +195,23 @@ case $1 in
    'check-url-access')   # check if we can access the URL - will exit 77 when not OK
 		rsyslog_testbench_test_url_access $2
 		;;
+   'check-command-available')   # check if command $2 is available - will exit 77 when not OK
+		command -v $2
+		if [ $? -ne 0 ] ; then
+			echo Testbench requires unavailable command: $2
+			exit 77
+		fi
+		;;
+   'check-ipv6-available')   # check if IPv6  - will exit 77 when not OK
+		ifconfig -a |grep ::1
+		if [ $? -ne 0 ] ; then
+			echo this test requires an active IPv6 stack, which we do not have here
+			exit 77
+		fi
+		;;
    'es-init')   # initialize local Elasticsearch *testbench* instance for the next
                 # test. NOTE: do NOT put anything useful on that instance!
-		curl -XDELETE localhost:9200/rsyslog_testbench
+		curl --silent -XDELETE localhost:${ES_PORT:-9200}/rsyslog_testbench
 		;;
    'es-getdata') # read data from ES to a local file so that we can process
 		if [ "x$3" == "x" ]; then
@@ -202,7 +223,7 @@ case $1 in
    		# it with out regular tooling.
 		# Note: param 2 MUST be number of records to read (ES does
 		# not return the full set unless you tell it explicitely).
-		curl localhost:$es_get_port/rsyslog_testbench/_search?size=$2 > work
+		curl --silent localhost:$es_get_port/rsyslog_testbench/_search?size=$2 > work
 		python $srcdir/es_response_get_msgnum.py > rsyslog.out.log
 		;;
    'getpid')
@@ -248,7 +269,7 @@ case $1 in
 		    echo "ERROR: config file '$CONF_FILE' not found!"
 		    exit 1
 		fi
-		valgrind $RS_TEST_VALGRIND_EXTRA_OPTS $RS_TESTBENCH_VALGRIND_EXTRA_OPTS --gen-suppressions=all --log-fd=1 --error-exitcode=10 --malloc-fill=ff --free-fill=fe --leak-check=$RS_TESTBENCH_LEAK_CHECK ../tools/rsyslogd -C -n -irsyslog$3.pid -M../runtime/.libs:../.libs -f$CONF_FILE &
+		LD_PRELOAD=$RSYSLOG_PRELOAD valgrind $RS_TEST_VALGRIND_EXTRA_OPTS $RS_TESTBENCH_VALGRIND_EXTRA_OPTS --gen-suppressions=all --log-fd=1 --error-exitcode=10 --malloc-fill=ff --free-fill=fe --leak-check=$RS_TESTBENCH_LEAK_CHECK ../tools/rsyslogd -C -n -irsyslog$3.pid -M../runtime/.libs:../.libs -f$CONF_FILE &
 		. $srcdir/diag.sh wait-startup-pid $3
 		;;
    'startup-vg') # start rsyslogd with default params under valgrind control. $2 is the config file name to use
@@ -325,9 +346,9 @@ case $1 in
 		done
 		echo "rsyslogd$2 startup msg seen, pid " `cat rsyslog$2.pid`
 		;;
-   'wait-pid-termination')  # wait for the pid in pid file $2 to terminate, abort on timeout
+   'wait-pid-termination')  # wait for the pid in pid $2 to terminate, abort on timeout
 		i=0
-		out_pid=`cat $2`
+		out_pid=$2
 		if [[ "x$out_pid" == "x" ]]
 		then
 			terminated=1
@@ -509,6 +530,17 @@ case $1 in
 		  . $srcdir/diag.sh error-exit 1
     fi
     ;;
+   'grep-check') # grep for "$EXPECTED" present in rsyslog.log - env var must be set
+		 # before this method is called
+		grep "$EXPECTED" rsyslog.out.log > /dev/null
+		if [ $? -eq 1 ]; then
+		  echo "GREP FAIL: rsyslog.out.log content:"
+		  cat rsyslog.out.log
+		  echo "GREP FAIL: expected text not found:"
+		  echo "$EXPECTED"
+		. $srcdir/diag.sh error-exit 1
+		fi;
+		;;
    'seq-check') # do the usual sequence check to see if everything was properly received. $2 is the instance.
 		rm -f work
 		cp rsyslog.out.log work-presort
@@ -535,7 +567,18 @@ case $1 in
 		fi
 		rm -f work2
 		;;
-   'content-check') 
+   'content-cmp')
+		echo "$2" | cmp - rsyslog.out.log
+		if [ "$?" -ne "0" ]; then
+		    echo content-cmp failed
+		    echo EXPECTED:
+		    echo $2
+		    echo ACTUAL:
+		    cat rsyslog.out.log
+		    . $srcdir/diag.sh error-exit 1
+		fi
+		;;
+   'content-check')
 		cat rsyslog.out.log | grep -qF "$2"
 		if [ "$?" -ne "0" ]; then
 		    echo content-check failed to find "'$2'", content is
@@ -543,6 +586,36 @@ case $1 in
 		    . $srcdir/diag.sh error-exit 1
 		fi
 		;;
+   'wait-file-lines') 
+		# $2 filename, $3 expected nbr of lines, $4 nbr of tries
+		if [ "$4" == "" ]; then
+			timeoutend=1
+		else
+			timeoutend=$4
+		fi
+		timecounter=0
+
+		while [  $timecounter -lt $timeoutend ]; do
+			let timecounter=timecounter+1
+
+			count=$(wc -l < rsyslog.out.log)
+			if [ $count -eq $3 ]; then
+				echo wait-file-lines success, have $3 lines
+				break
+			else
+				if [ "x$timecounter" == "x$timeoutend" ]; then
+					echo wait-file-lines failed, expected $3 got $count
+					. $srcdir/diag.sh shutdown-when-empty
+					. $srcdir/diag.sh wait-shutdown
+					. $srcdir/diag.sh error-exit 1
+				else
+					echo wait-file-lines not yet there, currently $count lines
+					./msleep 200
+				fi
+			fi
+		done
+		unset count
+		;;
    'content-check-with-count') 
 		# content check variables for Timeout
 		if [ "x$4" == "x" ]; then
@@ -553,12 +626,11 @@ case $1 in
 		timecounter=0
 
 		while [  $timecounter -lt $timeoutend ]; do
-#			echo content-check-with-count loop $timecounter
 			let timecounter=timecounter+1
 
 			count=$(cat rsyslog.out.log | grep -F "$2" | wc -l)
 
-			if [ "x$count" == "x$3" ]; then
+			if [ $count -eq $3 ]; then
 				echo content-check-with-count success, \"$2\" occured $3 times
 				break
 			else
@@ -566,12 +638,12 @@ case $1 in
 					. $srcdir/diag.sh shutdown-when-empty # shut down rsyslogd
 					. $srcdir/diag.sh wait-shutdown	# Shutdown rsyslog instance on error 
 
-					echo content-check-with-count failed, expected \"$2\" to occure $3 times, but found it $count times
+					echo content-check-with-count failed, expected \"$2\" to occur $3 times, but found it $count times
 					echo file rsyslog.out.log content is:
-					cat rsyslog.out.log
+					sort < rsyslog.out.log
 					. $srcdir/diag.sh error-exit 1
 				else
-					echo content-check-with-count failed, trying again ...
+					echo content-check-with-count have $count, wait for $3 times $2...
 					./msleep 1000
 				fi
 			fi
@@ -616,9 +688,9 @@ case $1 in
 		done
 		echo "dyn-stats reset for bucket ${3} registered"
 		;;
-   'content-check')
+   'content-check-regex')
 		# this does a content check which permits regex
-		grep "$2" $3
+		grep "$2" $3 -q
 		if [ "$?" -ne "0" ]; then
 		    echo "----------------------------------------------------------------------"
 		    echo content-check failed to find "'$2'" inside "'$3'"
@@ -861,14 +933,19 @@ case $1 in
 		fi
 		if [ -d $dep_work_dir/es ]; then
 			if [ -e $dep_work_es_pidfile ]; then
-				kill -SIGTERM $(cat $dep_work_es_pidfile)
-				. $srcdir/diag.sh wait-pid-termination $dep_work_es_pidfile
+				es_pid = $(cat $dep_work_es_pidfile)
+				kill -SIGTERM $es_pid
+				. $srcdir/diag.sh wait-pid-termination $es_pid
 			fi
 		fi
 		rm -rf $dep_work_dir/es
 		echo TEST USES ELASTICSEARCH BINARY $dep_es_cached_file
 		(cd $dep_work_dir && tar -zxf $dep_es_cached_file --xform $dep_es_dir_xform_pattern --show-transformed-names) > /dev/null
-		cp $srcdir/testsuites/$dep_work_es_config $dep_work_dir/es/config/elasticsearch.yml
+		if [ -n "${ES_PORT:-}" ] ; then
+			sed "s/^http.port:.*\$/http.port: ${ES_PORT}/" $srcdir/testsuites/$dep_work_es_config > $dep_work_dir/es/config/elasticsearch.yml
+		else
+			cp $srcdir/testsuites/$dep_work_es_config $dep_work_dir/es/config/elasticsearch.yml
+		fi
 
 		if [ ! -d $dep_work_dir/es/data ]; then
 				echo "Creating elastic search directories"
@@ -903,8 +980,8 @@ case $1 in
 		timeseconds=0
 		# Loop until elasticsearch port is reachable or until
 		# timeout is reached!
-		until [ "`curl --silent --show-error --connect-timeout 1 http://localhost:19200 | grep 'rsyslog-testbench'`" != "" ]; do
-			echo "--- waiting for startup: 1 ( $timeseconds ) seconds"
+		until [ "`curl --silent --show-error --connect-timeout 1 http://localhost:${ES_PORT:-19200} | grep 'rsyslog-testbench'`" != "" ]; do
+			echo "--- waiting for ES startup: $timeseconds seconds"
 			./msleep 1000
 			let "timeseconds = $timeseconds + 1"
 
@@ -914,6 +991,7 @@ case $1 in
 			fi
 		done
 		./msleep 2000
+		echo ES startup succeeded
 		;;
 	 'dump-kafka-serverlog')
 		if [ "x$2" == "x" ]; then
@@ -976,18 +1054,16 @@ case $1 in
 			dep_work_es_pidfile="es$2.pid"
 		fi
 		if [ -e $dep_work_es_pidfile ]; then
-			(cd $srcdir && kill $(cat $dep_work_es_pidfile) )
-			. $srcdir/diag.sh wait-pid-termination $dep_work_es_pidfile
+			es_pid=$(cat $dep_work_es_pidfile)
+			printf "stopping ES with pid %d\n" $es_pid
+			kill -SIGTERM $es_pid
+			. $srcdir/diag.sh wait-pid-termination $es_pid
 		fi
 		;;
 	 'cleanup-elasticsearch')
-		if [ "x$2" == "x" ]; then
-			dep_work_dir=$(readlink -f $srcdir/.dep_wrk)
-			dep_work_es_pidfile="es.pid"
-		else
-			dep_work_dir=$(readlink -f $srcdir/$2)
-			dep_work_es_pidfile="es$2.pid"
-		fi
+		dep_work_dir=$(readlink -f $srcdir/.dep_wrk)
+		dep_work_es_pidfile="es.pid"
+		. $srcdir/diag.sh stop-elasticsearch
 		rm -f $dep_work_es_pidfile
 		rm -rf $dep_work_dir/es
 		;;
@@ -1090,9 +1166,6 @@ case $1 in
 			gdb ../tools/rsyslogd $CORE -batch -x gdb.in
 			CORE=
 			rm gdb.in
-		else
-			echo no core file found, cannot provide additional info
-			ls -l core*
 		fi
 		if [[ "$3" == 'stacktrace' || ( ! -e IN_AUTO_DEBUG &&  "$USE_AUTO_DEBUG" == 'on' ) ]]; then
 			if [ -e core* ]
