@@ -1,12 +1,6 @@
 #!/bin/bash
 # addd 2016-07-11 by RGerhards, released under ASL 2.0
 
-uname
-if [ `uname` = "FreeBSD" ] ; then
-   echo "This test currently does not work on FreeBSD."
-   exit 77
-fi
-
 . $srcdir/diag.sh init
 . $srcdir/diag.sh generate-conf
 . $srcdir/diag.sh add-conf '
@@ -23,7 +17,7 @@ echo '<167>1 2003-03-01T01:00:00.000Z hostname1/hostname2 tcpflood - tag [tcpflo
 . $srcdir/diag.sh tcpflood -B -I rsyslog.input
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
-echo "hostname1/hostname2" | cmp rsyslog.out.log
+echo "hostname1/hostname2" | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "invalid hostname generated, rsyslog.out.log is:"
   cat rsyslog.out.log
