@@ -18,7 +18,7 @@ echo '<167>Mar  6 16:57:54 172.20.245.8 test: msgnum:0 X test message
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
 echo '<167>Mar  6 16:57:54 172.20.245.8 test: msgnum:0 #000 test message
-<167>Mar  6 16:57:54 172.20.245.8 #000test: msgnum:1 test message' | cmp rsyslog.out.log
+<167>Mar  6 16:57:54 172.20.245.8 #000test: msgnum:1 test message' | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "invalid output generated, rsyslog.out.log is:"
   cat rsyslog.out.log
