@@ -5,7 +5,8 @@
 . $srcdir/diag.sh generate-conf
 . $srcdir/diag.sh add-conf '
 $MaxMessageSize 128
-global(processInternalMessages="on")
+global(processInternalMessages="on"
+	oversizemsg.input.mode="accept")
 module(load="../plugins/imtcp/.libs/imtcp")
 input(type="imtcp" port="13514")
 
@@ -17,7 +18,7 @@ action(type="omfile" file="rsyslog.out.log")
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
 
-grep "Framing Error.*change to octet stuffing" rsyslog.out.log > /dev/null
+grep "Framing Error in received" rsyslog.out.log > /dev/null
 if [ $? -ne 0 ]; then
         echo
         echo "FAIL: expected error message from imtcp not found. rsyslog.out.log is:"
@@ -28,7 +29,7 @@ fi
 grep "9876543210cdefghijklmn test8 test9 test10 test11 test12 test13 test14 test15 kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk tag: testtestt" rsyslog.out.log > /dev/null
 if [ $? -ne 0 ]; then
         echo
-        echo "FAIL: expected error message from imtcp not found. rsyslog.out.log is:"
+        echo "FAIL: expected date from imtcp not found. rsyslog.out.log is:"
         cat rsyslog.out.log
         . $srcdir/diag.sh error-exit 1
 fi
