@@ -1,6 +1,13 @@
 #!/bin/bash
 # added 2015-09-30 by singh.janmejay
 # This file is part of the rsyslog project, released under ASL 2.0
+
+uname
+if [ `uname` = "FreeBSD" ] ; then
+   echo "This test currently does not work on FreeBSD."
+   exit 77
+fi
+
 echo ===============================================================================
 echo \[lookup_table_no_hup_reload.sh\]: test for lookup-table with HUP based reloading disabled
 . $srcdir/diag.sh init
