@@ -1,5 +1,12 @@
 #!/bin/bash
 # test many concurrent tcp connections
+
+uname
+if [ `uname` = "FreeBSD" ] ; then
+   echo "This test currently does not work on FreeBSD."
+   exit 77
+fi
+
 echo \[manytcp.sh\]: test concurrent tcp connections
 
 uname
