@@ -15,7 +15,7 @@ FAKETIME='2016-01-01 01:00:00' $srcdir/diag.sh startup now_family_utc.conf
 . $srcdir/diag.sh tcpflood -m1
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
-echo "01:00,07:30" | cmp rsyslog.out.log
+echo "01:00,07:30" | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "invalid timestamps generated, rsyslog.out.log is:"
   cat rsyslog.out.log
