@@ -1,12 +1,6 @@
 #!/bin/bash
 # add 2016-11-22 by Pascal Withopf, released under ASL 2.0
 
-uname
-if [ `uname` = "FreeBSD" ] ; then
-   echo "This test currently does not work on FreeBSD."
-   exit 77
-fi
-
 . $srcdir/diag.sh init
 . $srcdir/diag.sh generate-conf
 . $srcdir/diag.sh add-conf '
@@ -34,7 +28,7 @@ if [ $? -ne 0 ]; then
 	. $srcdir/diag.sh error-exit 1
 fi
 
-echo '- msgnum:1-' | cmp rsyslog.out.log
+echo '- msgnum:1-' | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "unexpected content in rsyslog.out.log is:"
   cat rsyslog.out.log
