@@ -1,3 +1,4 @@
+#!/bin/bash
 # Basic test for omruleset. What we do is have the main queue forward 
 # all messages to a secondary ruleset via omruleset, which then does
 # the actual file write. We check if all messages arrive at the file, 
@@ -11,12 +12,12 @@
 # This file is part of the rsyslog project, released under GPLv3
 echo ===============================================================================
 echo \[omruleset.sh\]: basic test for omruleset functionality
-source $srcdir/diag.sh init
-source $srcdir/diag.sh startup omruleset.conf
-source $srcdir/diag.sh injectmsg  0 5000
+. $srcdir/diag.sh init
+. $srcdir/diag.sh startup omruleset.conf
+. $srcdir/diag.sh injectmsg  0 5000
 echo doing shutdown
-source $srcdir/diag.sh shutdown-when-empty
+. $srcdir/diag.sh shutdown-when-empty
 echo wait on shutdown
-source $srcdir/diag.sh wait-shutdown 
-source $srcdir/diag.sh seq-check  0 4999
-source $srcdir/diag.sh exit
+. $srcdir/diag.sh wait-shutdown 
+. $srcdir/diag.sh seq-check  0 4999
+. $srcdir/diag.sh exit
