@@ -1,12 +1,6 @@
 #!/bin/bash
 # addd 2017-01142 by RGerhards, released under ASL 2.0
 
-uname
-if [ `uname` = "FreeBSD" ] ; then
-   echo "This test currently does not work on FreeBSD."
-   exit 77
-fi
-
 . $srcdir/diag.sh init
 . $srcdir/diag.sh generate-conf
 . $srcdir/diag.sh add-conf '
@@ -23,7 +17,7 @@ local0.* action(type="omfile" template="outfmt"
 . $srcdir/diag.sh tcpflood -m1
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
-echo "tag/with/slashes,tag/with/slashes" | cmp rsyslog.out.log
+echo "tag/with/slashes,tag/with/slashes" | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "invalid output generated, rsyslog.out.log is:"
   cat rsyslog.out.log
