@@ -1,13 +1,14 @@
+#!/bin/bash
 # added 2014-01-17 by rgerhards
 # This file is part of the rsyslog project, released under ASL 2.0
 echo ===============================================================================
 echo \[rscript_gt.sh\]: testing rainerscript GT statement
-source $srcdir/diag.sh init
-source $srcdir/diag.sh startup rscript_gt.conf
-source $srcdir/diag.sh injectmsg  0 8000
+. $srcdir/diag.sh init
+. $srcdir/diag.sh startup rscript_gt.conf
+. $srcdir/diag.sh injectmsg  0 8000
 echo doing shutdown
-source $srcdir/diag.sh shutdown-when-empty
+. $srcdir/diag.sh shutdown-when-empty
 echo wait on shutdown
-source $srcdir/diag.sh wait-shutdown 
-source $srcdir/diag.sh seq-check  5000 7999
-source $srcdir/diag.sh exit
+. $srcdir/diag.sh wait-shutdown 
+. $srcdir/diag.sh seq-check  5000 7999
+. $srcdir/diag.sh exit
