@@ -1,12 +1,5 @@
 #!/bin/bash
 # addd 2016-03-22 by RGerhards, released under ASL 2.0
-
-uname
-if [ `uname` = "FreeBSD" ] ; then
-   echo "This test currently does not work on FreeBSD."
-   exit 77
-fi
-
 . $srcdir/diag.sh init
 . $srcdir/diag.sh generate-conf
 . $srcdir/diag.sh add-conf '
@@ -25,7 +18,7 @@ rm -f rsyslog.out.log	# do cleanup of previous subtest
 . $srcdir/diag.sh tcpflood -m1 -M"\"<165>1 2003-08-24T05:14:15.000003-07:00 192.0.2.1 tcpflood 8710 - - msgnum:0000000\""
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
-echo "2003-08-24T12:14:15.000003+00:00" | cmp rsyslog.out.log
+echo "2003-08-24T12:14:15.000003+00:00" | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "invalid timestamps generated, rsyslog.out.log is:"
   cat rsyslog.out.log
@@ -38,7 +31,7 @@ rm -f rsyslog.out.log	# do cleanup of previous subtest
 . $srcdir/diag.sh tcpflood -m1 -M"\"<165>1 2016-03-01T12:00:00-02:00 192.0.2.1 tcpflood 8710 - - msgnum:0000000\""
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
-echo "2016-03-01T14:00:00.000000+00:00" | cmp rsyslog.out.log
+echo "2016-03-01T14:00:00.000000+00:00" | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "invalid timestamps generated, rsyslog.out.log is:"
   cat rsyslog.out.log
@@ -51,7 +44,7 @@ rm -f rsyslog.out.log	# do cleanup of previous subtest
 . $srcdir/diag.sh tcpflood -m1 -M"\"<165>1 2016-03-01T12:00:00Z 192.0.2.1 tcpflood 8710 - - msgnum:0000000\""
 . $srcdir/diag.sh shutdown-when-empty
 . $srcdir/diag.sh wait-shutdown
-echo "2016-03-01T12:00:00.000000+00:00" | cmp rsyslog.out.log
+echo "2016-03-01T12:00:00.000000+00:00" | cmp - rsyslog.out.log
 if [ ! $? -eq 0 ]; then
   echo "invalid timestamps generated, rsyslog.out.log is:"
   cat rsyslog.out.log
