@@ -3,7 +3,7 @@
  * because it was either written from scratch by me (rgerhards) or
  * contributors who agreed to ASL 2.0.
  *
- * Copyright 2004-2017 Rainer Gerhards and Adiscon
+ * Copyright 2004-2018 Rainer Gerhards and Adiscon
  *
  * This file is part of rsyslog.
  *
@@ -29,16 +29,12 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include <errno.h>
 #ifdef HAVE_LIBLOGGING_STDLOG
 #  include <liblogging/stdlog.h>
 #else
 #  include <syslog.h>
 #endif
-#if defined(OS_SOLARIS) || defined(OS_BSD)
-#	include <errno.h>
-#else
-#	include <sys/errno.h>
-#endif
 #ifdef HAVE_LIBSYSTEMD
 #	include <systemd/sd-daemon.h>
 #endif
@@ -123,7 +119,6 @@ DEFobjCurrIf(prop)
 DEFobjCurrIf(parser)
 DEFobjCurrIf(ruleset)
 DEFobjCurrIf(net)
-DEFobjCurrIf(errmsg)
 DEFobjCurrIf(rsconf)
 DEFobjCurrIf(module)
 DEFobjCurrIf(datetime)
@@ -160,7 +155,7 @@ int bFinished = 0;	/* used by termination signal handler, read-only except there
 			 * is either 0 or the number of the signal that requested the
  			 * termination.
 			 */
-const char *PidFile = PATH_PIDFILE;
+const char *PidFile;
 #define NO_PIDFILE "NONE"
 int iConfigVerify = 0;	/* is this just a config verify run? */
 rsconf_t *ourConf = NULL;	/* our config object */
@@ -187,7 +182,7 @@ rsyslogd_usage(void)
 {
 	fprintf(stderr, "usage: rsyslogd [options]\n"
 			"use \"man rsyslogd\" for details. To run rsyslog "
-			"interactively, use \"rsyslogd -n\""
+			"interactively, use \"rsyslogd -n\"\n"
 			"to run it in debug mode use \"rsyslogd -dn\"\n"
 			"For further information see http://www.rsyslog.com/doc\n");
 	exit(1); /* "good" exit - done to terminate usage() */
@@ -280,7 +275,7 @@ writePidFile(void)
 		ABORT_FINALIZE(RS_RET_ERR);
 	}
 	if(fprintf(fp, "%d", (int) glblGetOurPid()) < 0) {
-		errmsg.LogError(errno, iRet, "rsyslog: error writing pid file");
+		LogError(errno, iRet, "rsyslog: error writing pid file");
 	}
 	fclose(fp);
 	if(tmpPidFile != PidFile) {
@@ -595,8 +590,6 @@ rsyslogd_InitGlobalClasses(void)
 	/* Now tell the system which classes we need ourselfs */
 	pErrObj = "glbl";
 	CHKiRet(objUse(glbl,     CORE_COMPONENT));
-	pErrObj = "errmsg";
-	CHKiRet(objUse(errmsg,   CORE_COMPONENT));
 	pErrObj = "module";
 	CHKiRet(objUse(module,   CORE_COMPONENT));
 	pErrObj = "datetime";
@@ -734,7 +727,7 @@ rsRetVal createMainQueue(qqueue_t **ppQueue, uchar *pszQueueName, struct nvlst *
 	CHKiRet_Hdlr(qqueueConstruct(ppQueue, ourConf->globals.mainQ.MainMsgQueType,
 	ourConf->globals.mainQ.iMainMsgQueueNumWorkers, ourConf->globals.mainQ.iMainMsgQueueSize, msgConsumer)) {
 		/* no queue is fatal, we need to give up in that case... */
-		errmsg.LogError(0, iRet, "could not create (ruleset) main message queue"); \
+		LogError(0, iRet, "could not create (ruleset) main message queue"); \
 	}
 	/* name our main queue object (it's not fatal if it fails...) */
 	obj.SetName((obj_t*) (*ppQueue), pszQueueName);
@@ -743,12 +736,12 @@ rsRetVal createMainQueue(qqueue_t **ppQueue, uchar *pszQueueName, struct nvlst *
 		/* ... set some properties ... */
 	#	define setQPROP(func, directive, data) \
 		CHKiRet_Hdlr(func(*ppQueue, data)) { \
-			errmsg.LogError(0, NO_ERRCODE, "Invalid " #directive ", error %d. Ignored, " \
+			LogError(0, NO_ERRCODE, "Invalid " #directive ", error %d. Ignored, " \
 			"running with default setting", iRet); \
 		}
 	#	define setQPROPstr(func, directive, data) \
 		CHKiRet_Hdlr(func(*ppQueue, data, (data == NULL)? 0 : strlen((char*) data))) { \
-			errmsg.LogError(0, NO_ERRCODE, "Invalid " #directive ", error %d. Ignored, " \
+			LogError(0, NO_ERRCODE, "Invalid " #directive ", error %d. Ignored, " \
 			"running with default setting", iRet); \
 		}
 
@@ -762,7 +755,7 @@ rsRetVal createMainQueue(qqueue_t **ppQueue, uchar *pszQueueName, struct nvlst *
 						 ++qfn_renamenum, ourConf->globals.mainQ.pszMainMsgQFName,  
 						 (pszQueueName == NULL) ? "NONAME" : (char*)pszQueueName);
 					qfname = ustrdup(qfrenamebuf);
-					errmsg.LogError(0, NO_ERRCODE, "Error: queue file name '%s' already in use "
+					LogError(0, NO_ERRCODE, "Error: queue file name '%s' already in use "
 						" - using '%s' instead", ourConf->globals.mainQ.pszMainMsgQFName,
 						qfname);
 					break;
@@ -828,7 +821,7 @@ startMainQueue(qqueue_t *pQueue)
 	DEFiRet;
 	CHKiRet_Hdlr(qqueueStart(pQueue)) {
 		/* no queue is fatal, we need to give up in that case... */
-		errmsg.LogError(0, iRet, "could not start (ruleset) main message queue"); \
+		LogError(0, iRet, "could not start (ruleset) main message queue"); \
 	}
 	RETiRet;
 }
@@ -866,6 +859,19 @@ logmsgInternal_doWrite(smsg_t *pMsg)
 		const int pri = getPRIi(pMsg);
 		uchar *const msg = getMSG(pMsg);
 #		ifdef HAVE_LIBLOGGING_STDLOG
+		/* the "emit only once" rate limiter is quick and dirty and not
+		 * thread safe. However, that's no problem for the current intend
+		 * and it is not justified to create more robust code for the
+		 * functionality. -- rgerhards, 2018-05-14
+		 */
+		static warnmsg_emitted = 0;
+		if(warnmsg_emitted == 0) {
+			stdlog_log(stdlog_hdl, LOG_WARNING, "%s",
+				"RSYSLOG WARNING: liblogging-stdlog "
+				"functionality will go away soon. For details see "
+				"https://github.com/rsyslog/rsyslog/issues/2706");
+			warnmsg_emitted = 1;
+		}
 		stdlog_log(stdlog_hdl, pri2sev(pri), "%s", (char*)msg);
 #		else
 		syslog(pri, "%s", msg);
@@ -989,6 +995,45 @@ submitMsg(smsg_t *pMsg)
 	return submitMsgWithDfltRatelimiter(pMsg);
 }
 
+
+static rsRetVal ATTR_NONNULL()
+splitOversizeMessage(smsg_t *const pMsg)
+{
+	DEFiRet;
+	const char *rawmsg;
+	int nsegments;
+	int len_rawmsg;
+	const int maxlen = glblGetMaxLine();
+	ISOBJ_TYPE_assert(pMsg, msg);
+
+	getRawMsg(pMsg, (uchar**) &rawmsg, &len_rawmsg);
+	nsegments = len_rawmsg / maxlen;
+	const int len_last_segment = len_rawmsg % maxlen;
+	DBGPRINTF("splitting oversize message, size %d, segment size %d, "
+		"nsegments %d, bytes in last fragment %d\n",
+		len_rawmsg, maxlen, nsegments, len_last_segment);
+
+	smsg_t *pMsg_seg;
+
+	/* process full segments */
+	for(int i = 0 ; i < nsegments ; ++i) {
+		CHKmalloc(pMsg_seg = MsgDup(pMsg));
+		MsgSetRawMsg(pMsg_seg, rawmsg + (i * maxlen), maxlen);
+		submitMsg2(pMsg_seg);
+	}
+
+	/* if necessary, write partial last segment */
+	if(len_last_segment != 0) {
+		CHKmalloc(pMsg_seg = MsgDup(pMsg));
+		MsgSetRawMsg(pMsg_seg, rawmsg + (nsegments * maxlen), len_last_segment);
+		submitMsg2(pMsg_seg);
+	}
+
+finalize_it:
+	RETiRet;
+}
+
+
 /* submit a message to the main message queue.   This is primarily
  * a hook to prevent the need for callers to know about the main message queue
  * rgerhards, 2008-02-13
@@ -1002,6 +1047,34 @@ submitMsg2(smsg_t *pMsg)
 
 	ISOBJ_TYPE_assert(pMsg, msg);
 
+	if(getRawMsgLen(pMsg) > glblGetMaxLine()){
+		uchar *rawmsg;
+		int dummy;
+		getRawMsg(pMsg, &rawmsg, &dummy);
+		if(glblReportOversizeMessage()) {
+			LogMsg(0, RS_RET_OVERSIZE_MSG, LOG_WARNING,
+				"message too long (%d) with configured size %d, begin of "
+				"message is: %.80s",
+				getRawMsgLen(pMsg), glblGetMaxLine(), rawmsg);
+		}
+		writeOversizeMessageLog(pMsg);
+		if(glblGetOversizeMsgInputMode() == glblOversizeMsgInputMode_Split) {
+			splitOversizeMessage(pMsg);
+			/* we have submitted the message segments recursively, so we
+			 * can just deleted the original msg object and terminate.
+			 */
+			msgDestruct(&pMsg);
+			FINALIZE;
+		} else if(glblGetOversizeMsgInputMode() == glblOversizeMsgInputMode_Truncate) {
+			MsgTruncateToMaxSize(pMsg);
+		} else {
+			/* in "accept" mode, we do nothing, simply because "accept" means
+			 * to use as-is.
+			 */
+			assert(glblGetOversizeMsgInputMode() == glblOversizeMsgInputMode_Accept);
+		}
+	}
+
 	pRuleset = MsgGetRuleset(pMsg);
 	pQueue = (pRuleset == NULL) ? pMsgQueue : ruleset.GetRulesetQueue(pRuleset);
 
@@ -1022,13 +1095,12 @@ finalize_it:
  * for multi_submit_t. All messages need to go into the SAME queue!
  * rgerhards, 2009-06-16
  */
-rsRetVal
-multiSubmitMsg2(multi_submit_t *pMultiSub)
+rsRetVal ATTR_NONNULL()
+multiSubmitMsg2(multi_submit_t *const pMultiSub)
 {
 	qqueue_t *pQueue;
 	ruleset_t *pRuleset;
 	DEFiRet;
-	assert(pMultiSub != NULL);
 
 	if(pMultiSub->nElem == 0)
 		FINALIZE;
@@ -1305,6 +1377,16 @@ initAll(int argc, char **argv)
 	 */
 	queryLocalHostname();
 
+	/* we now can emit error messages "the regular way" */
+
+	if(getenv("TZ") == NULL) {
+		const char *const tz =
+			(access("/etc/localtime", R_OK) == 0) ? "TZ=/etc/localtime" : "TZ=UTC";
+		putenv((char*)tz);
+		LogMsg(0, RS_RET_NO_TZ_SET, LOG_WARNING, "environment variable TZ is not "
+			"set, auto correcting this to %s\n", tz);
+	}
+
 	/* END core initializations - we now come back to carrying out command line options*/
 
 	while((iRet = bufOptRemove(&ch, &arg)) == RS_RET_OK) {
@@ -1342,6 +1424,7 @@ initAll(int argc, char **argv)
 			ConfFile = (uchar*) arg;
 			break;
 		case 'i':		/* pid file name */
+			free((void*)PidFile);
 			PidFile = arg;
 			break;
 		case 'l':
@@ -1448,6 +1531,23 @@ initAll(int argc, char **argv)
 
 	resetErrMsgsFlag();
 	localRet = rsconf.Load(&ourConf, ConfFile);
+
+	/* check for "hard" errors that needs us to abort in any case */
+	if(   (localRet == RS_RET_CONF_FILE_NOT_FOUND)
+	   || (localRet == RS_RET_NO_ACTIONS) ) {
+		/* for extreme testing, we keep the ability to let rsyslog continue
+		 * even on hard config errors. Note that this may lead to segfaults
+		 * or other malfunction further down the road.
+		 */
+		if((glblDevOptions & DEV_OPTION_KEEP_RUNNING_ON_HARD_CONF_ERROR) == 1) {
+			fprintf(stderr, "rsyslogd: NOTE: developer-only option set to keep rsyslog "
+				"running where it should abort - this can lead to "
+				"more problems later in the run.\n");
+		} else {
+			ABORT_FINALIZE(localRet);
+		}
+	}
+
 	glbl.GenerateLocalHostNameProperty();
 
 	if(hadErrMsgs()) {
@@ -1486,7 +1586,7 @@ initAll(int argc, char **argv)
 
 	hdlr_enable(SIGPIPE, SIG_IGN);
 	hdlr_enable(SIGXFSZ, SIG_IGN);
-	if(Debug) {
+	if(Debug || glblPermitCtlC) {
 		hdlr_enable(SIGUSR1, rsyslogdDebugSwitch);
 		hdlr_enable(SIGINT,  rsyslogdDoDie);
 		hdlr_enable(SIGQUIT, rsyslogdDoDie);
@@ -1658,6 +1758,7 @@ doHUP(void)
 	ruleset.IterateAllActions(ourConf, doHUPActions, NULL);
 	modDoHUP();
 	lookupDoHUP();
+	errmsgDoHUP();
 }
 
 /* rsyslogdDoDie() is a signal handler. If called, it sets the bFinished variable
@@ -1947,6 +2048,22 @@ main(int argc, char **argv)
 		}
 #endif
 
+	if((int) getpid() == 1) {
+		fprintf(stderr, "rsyslogd %s: running as pid 1, enabling "
+			"container-specific defaults, press ctl-c to "
+			"terminate rsyslog\n", VERSION);
+		PidFile = strdup("NONE"); /* disables pid file writing */
+		glblPermitCtlC = 1;
+	} else {
+		/* "dynamic defaults" - non-container case */
+		PidFile = strdup(PATH_PIDFILE);
+	}
+	if(PidFile == NULL) {
+		fprintf(stderr, "rsyslogd: could not alloc memory for pid file "
+			"default name - aborting\n");
+		exit(1);
+	}
+
 	/* disable case-sensitive comparisons in variable subsystem: */
 	fjson_global_do_case_sensitive_comparison(0);
 
